/* Sistemas Integrados TECNISA
   Servios

Nome: LIMPA_FINANCEIRO

Descrio:
   Apaga os registros das tabelas do financeiro

Criao:
 19/03/02 - Jenilson Ferreira

*/

IF EXISTS (SELECT
              *
           FROM SYSOBJECTS O, SYSUSERS U
           WHERE
              O.NAME = 'LIMPA_FINANCEIRO' AND
              TYPE   = 'P' 	          AND
              U.UID  = O.UID              AND
              U.NAME = USER)

DROP PROCEDURE LIMPA_FINANCEIRO
GO

CREATE PROCEDURE LIMPA_FINANCEIRO
   @P_CDEMPRESA VARCHAR(02),
   @P_DTLIMITE  DATETIME

AS
   DECLARE @V_CTFILIAL            INTEGER;
   DECLARE @V_CTBAIXA             INTEGER;
   DECLARE @V_CTDEBI              INTEGER;
   DECLARE @V_CTCRED              INTEGER;
   DECLARE @V_CTTRANSF            INTEGER;
   DECLARE @V_DTBAIXA             DATETIME;
   DECLARE @V_DTBOLETO            DATETIME;
   DECLARE @V_CTBOL               INTEGER;

   DECLARE @V_BOLPAG_CDFILIAL     VARCHAR(04);
   DECLARE @V_BOLPAG_NRLANBOLPAG  VARCHAR(10);
   DECLARE @V_BOLPAG_CDFILIBAIPAG VARCHAR(04);
   DECLARE @V_BOLPAG_NRBAIXAPAG   VARCHAR(10);

   DECLARE @V_CNTPAG_CDFILIAL     VARCHAR(04);
   DECLARE @V_CNTPAG_NRLANCTONF   VARCHAR(10);
   DECLARE @V_CNTPAG_NRSEQVENPAG  VARCHAR(02);
   DECLARE @V_CNTPAG_NRPARCELAPAG VARCHAR(02);

   DECLARE @V_BOLREC_CDFILIAL     VARCHAR(04);
   DECLARE @V_BOLREC_NRLANBOLREC  VARCHAR(10);
   DECLARE @V_BOLREC_CDFILIBAIREC VARCHAR(04);
   DECLARE @V_BOLREC_NRBAIXAREC   VARCHAR(10);

   DECLARE @V_CNTREC_CDFILIAL     VARCHAR(04);
   DECLARE @V_CNTREC_NRLANCTONF   VARCHAR(10);
   DECLARE @V_CNTREC_NRSEQVENREC  VARCHAR(02);
   DECLARE @V_CNTREC_NRPARCELAREC VARCHAR(02);

   DECLARE @V_BAIXAPAG_CDFILIAL   VARCHAR(04);
   DECLARE @V_BAIXAPAG_NRBAIXAPAG VARCHAR(10);
   DECLARE @V_BAIXAPAG_CDREDUCONT VARCHAR(03);
   DECLARE @V_BAIXAPAG_NRMOVBANC  VARCHAR(10);

   DECLARE @V_BAIXAREC_CDFILIAL   VARCHAR(04);
   DECLARE @V_BAIXAREC_NRBAIXAREC VARCHAR(10);
   DECLARE @V_BAIXAREC_CDREDUCONT VARCHAR(03);
   DECLARE @V_BAIXAREC_NRMOVBANC  VARCHAR(10);

   DECLARE @V_MOVDEBI_CDREDUCONT  VARCHAR(03);
   DECLARE @V_MOVDEBI_NRMOVIDEBI  VARCHAR(10);
   DECLARE @V_MOVDEBI_NRMOVBANC   VARCHAR(10);

   DECLARE @V_MOVCRED_CDREDUCONT  VARCHAR(03);
   DECLARE @V_MOVCRED_NRMOVICRED  VARCHAR(10);
   DECLARE @V_MOVCRED_NRMOVBANC   VARCHAR(10);

   DECLARE @V_TRANSF_CDEMPRESA    VARCHAR(02);
   DECLARE @V_TRANSF_NRTRANBANC   VARCHAR(10);
   DECLARE @V_TRANSF_CDREDUCONT   VARCHAR(03);
   DECLARE @V_TRANSF_NRMOVBANC    VARCHAR(10);

BEGIN

/*  EXCLUSO DAS CONTAS A PAGAR E BOLETOS  */
   SELECT
      @V_CTFILIAL = COUNT(CDFILIAL)
   FROM
      BOLETPAGAR
   WHERE
      (CDEMPRESA    = @P_CDEMPRESA) AND
      (DTVENBOLPAG <= @P_DTLIMITE);

   IF @V_CTFILIAL > 0 BEGIN

      DECLARE C_BOLPAG CURSOR FOR
      SELECT CDFILIAL, NRLANBOLPAG, CDFILIBAIPAG, NRBAIXAPAG
      FROM BOLETPAGAR
      WHERE (CDEMPRESA    = @P_CDEMPRESA)  AND
            (DTVENBOLPAG <= @P_DTLIMITE);
      OPEN C_BOLPAG;
      FETCH NEXT FROM C_BOLPAG INTO @V_BOLPAG_CDFILIAL,     @V_BOLPAG_NRLANBOLPAG,
                                    @V_BOLPAG_CDFILIBAIPAG, @V_BOLPAG_NRBAIXAPAG;

      WHILE (@@FETCH_STATUS = 0) BEGIN
          IF @V_BOLPAG_NRBAIXAPAG IS NOT NULL BEGIN
             /* VERIFICA SE O BOLETO FOI BAIXADO EM UMA DATA POSTERIOR A DATA LIMITE */
             SELECT
                @V_DTBAIXA = DTBAIXAPAG
             FROM BAIXAPAGA
             WHERE
                (CDFILIAL   = @V_BOLPAG_CDFILIBAIPAG) AND
                (NRBAIXAPAG = @V_BOLPAG_NRBAIXAPAG);

             /* VERIFICA SE O BOLETO PERTENCE A UMA BAIXA QUE POSSUI ALGUM BOLETO COM VENCIMENTO POSTERIOR A DATA DA BAIXA */
             SELECT
                @V_DTBOLETO = MAX(DTVENBOLPAG)
             FROM BOLETPAGAR
             WHERE
                (CDFILIBAIPAG = @V_BOLPAG_CDFILIBAIPAG) AND
                (NRBAIXAPAG   = @V_BOLPAG_NRBAIXAPAG);
          END;

          IF (@V_BOLPAG_NRBAIXAPAG IS NULL) OR
             ((@V_DTBAIXA  <= @P_DTLIMITE)  AND
              (@V_DTBOLETO <= @P_DTLIMITE)) BEGIN

             DECLARE C_CNTPAG CURSOR FOR
             SELECT CDFILIAL, NRLANCTONF, NRSEQVENPAG, NRPARCELAPAG
             FROM CONTAPAGAR
             WHERE (CDFILIBPAG  = @V_BOLPAG_CDFILIAL) AND
                   (NRLANBOLPAG = @V_BOLPAG_NRLANBOLPAG);
             OPEN C_CNTPAG;
             FETCH NEXT FROM C_CNTPAG INTO @V_CNTPAG_CDFILIAL, @V_CNTPAG_NRLANCTONF,
                                           @V_CNTPAG_NRSEQVENPAG, @V_CNTPAG_NRPARCELAPAG;

             WHILE (@@FETCH_STATUS = 0) BEGIN
                  DELETE FROM RATCTPAGAR
                  WHERE
                     (CDFILIAL     = @V_CNTPAG_CDFILIAL)    AND
                     (NRLANCTONF   = @V_CNTPAG_NRLANCTONF)  AND
                     (NRSEQVENPAG  = @V_CNTPAG_NRSEQVENPAG) AND
                     (NRPARCELAPAG = @V_CNTPAG_NRPARCELAPAG);

                  DELETE FROM AJCTPAGAR
                  WHERE
                     (CDFILIAL     = @V_CNTPAG_CDFILIAL)    AND
                     (NRLANCTONF   = @V_CNTPAG_NRLANCTONF)  AND
                     (NRSEQVENPAG  = @V_CNTPAG_NRSEQVENPAG) AND
                     (NRPARCELAPAG = @V_CNTPAG_NRPARCELAPAG);

                  DELETE FROM CONTAPAGAR
                  WHERE
                     (CDFILIAL     = @V_CNTPAG_CDFILIAL)    AND
                     (NRLANCTONF   = @V_CNTPAG_NRLANCTONF)  AND
                     (NRSEQVENPAG  = @V_CNTPAG_NRSEQVENPAG) AND
                     (NRPARCELAPAG = @V_CNTPAG_NRPARCELAPAG);
                  FETCH NEXT FROM C_CNTPAG INTO @V_CNTPAG_CDFILIAL, @V_CNTPAG_NRLANCTONF,
                                                @V_CNTPAG_NRSEQVENPAG, @V_CNTPAG_NRPARCELAPAG;
             END;
             CLOSE C_CNTPAG;
             DEALLOCATE C_CNTPAG;

             DELETE FROM AJBOLPAG
             WHERE
                (CDFILIAL    = @V_BOLPAG_CDFILIAL) AND
                (NRLANBOLPAG = @V_BOLPAG_NRLANBOLPAG);

             DELETE FROM ERRCNABBOLP
             WHERE
                (CDFILIAL    = @V_BOLPAG_CDFILIAL) AND
                (NRLANBOLPAG = @V_BOLPAG_NRLANBOLPAG);

             DELETE FROM BOLETPAGAR
             WHERE
                (CDFILIAL    = @V_BOLPAG_CDFILIAL) AND
                (NRLANBOLPAG = @V_BOLPAG_NRLANBOLPAG);
          END;
          FETCH NEXT FROM C_BOLPAG INTO @V_BOLPAG_CDFILIAL,     @V_BOLPAG_NRLANBOLPAG,
                                        @V_BOLPAG_CDFILIBAIPAG, @V_BOLPAG_NRBAIXAPAG;
      END;
      CLOSE C_BOLPAG;
      DEALLOCATE C_BOLPAG;
   END;

/*  EXCLUSO DAS CONTAS A RECEBER E BOLETOS  */

   SELECT
      @V_CTFILIAL = COUNT(CDFILIAL)
   FROM
      BOLETRECEBER
   WHERE
      (CDEMPRESA    = @P_CDEMPRESA) AND
      (DTVENBOLREC <= @P_DTLIMITE);

   IF @V_CTFILIAL > 0 BEGIN
      DECLARE C_BOLREC CURSOR FOR
      SELECT CDFILIAL, NRLANBOLREC, CDFILIBAIREC, NRBAIXAREC
      FROM BOLETRECEBER
      WHERE (CDEMPRESA    = @P_CDEMPRESA) AND
            (DTVENBOLREC <= @P_DTLIMITE);
      OPEN C_BOLREC;
      FETCH NEXT FROM C_BOLREC INTO @V_BOLREC_CDFILIAL,     @V_BOLREC_NRLANBOLREC,
                                    @V_BOLREC_CDFILIBAIREC, @V_BOLREC_NRBAIXAREC;

      WHILE (@@FETCH_STATUS = 0) BEGIN
          IF @V_BOLREC_NRBAIXAREC IS NOT NULL BEGIN
             /* VERIFICA SE O BOLETO FOI BAIXADO EM UMA DATA POSTERIOR A DATA LIMITE */
             SELECT
                @V_DTBAIXA = DTBAIXAREC
             FROM BAIXARECEBIDA
             WHERE
                (CDFILIAL   = @V_BOLREC_CDFILIBAIREC) AND
                (NRBAIXAREC = @V_BOLREC_NRBAIXAREC);

             /* VERIFICA SE O BOLETO PERTENCE A UMA BAIXA QUE POSSUI ALGUM BOLETO COM VENCIMENTO POSTERIOR A DATA DA BAIXA */
             SELECT
                @V_DTBOLETO = MAX(DTVENBOLREC)
             FROM BOLETRECEBER
             WHERE
                (CDFILIBAIREC = @V_BOLREC_CDFILIBAIREC) AND
                (NRBAIXAREC   = @V_BOLREC_NRBAIXAREC);
          END;

          IF (@V_BOLREC_NRBAIXAREC IS NULL) OR
             ((@V_DTBAIXA  <= @P_DTLIMITE)  AND
              (@V_DTBOLETO <= @P_DTLIMITE)) BEGIN

             DECLARE C_CNTREC CURSOR FOR
             SELECT CDFILIAL, NRLANCTONF, NRSEQVENREC, NRPARCELAREC
             FROM CONTARECEBER
             WHERE (CDFILIBOLREC = @V_BOLREC_CDFILIAL)  AND
                   (NRLANBOLREC  = @V_BOLREC_NRLANBOLREC);
             OPEN C_CNTREC;
             FETCH NEXT FROM C_CNTREC INTO @V_CNTREC_CDFILIAL,    @V_CNTREC_NRLANCTONF,
                                           @V_CNTREC_NRSEQVENREC, @V_CNTREC_NRPARCELAREC;

             WHILE (@@FETCH_STATUS = 0) BEGIN
                DELETE FROM RATCTRECEBER
                WHERE
                   (CDFILIAL     = @V_CNTREC_CDFILIAL)    AND
                   (NRLANCTONF   = @V_CNTREC_NRLANCTONF)  AND
                   (NRSEQVENREC  = @V_CNTREC_NRSEQVENREC) AND
                   (NRPARCELAREC = @V_CNTREC_NRPARCELAREC);

                DELETE FROM AJCTRECEBER
                WHERE
                   (CDFILIAL     = @V_CNTREC_CDFILIAL)    AND
                   (NRLANCTONF   = @V_CNTREC_NRLANCTONF)  AND
                   (NRSEQVENREC  = @V_CNTREC_NRSEQVENREC) AND
                   (NRPARCELAREC = @V_CNTREC_NRPARCELAREC);

                DELETE FROM CONTARECEBER
                WHERE
                   (CDFILIAL     = @V_CNTREC_CDFILIAL)    AND
                   (NRLANCTONF   = @V_CNTREC_NRLANCTONF)  AND
                   (NRSEQVENREC  = @V_CNTREC_NRSEQVENREC) AND
                   (NRPARCELAREC = @V_CNTREC_NRPARCELAREC);
                FETCH NEXT FROM C_CNTREC INTO @V_CNTREC_CDFILIAL,    @V_CNTREC_NRLANCTONF,
                                              @V_CNTREC_NRSEQVENREC, @V_CNTREC_NRPARCELAREC;
             END;
             CLOSE C_CNTREC;
             DEALLOCATE C_CNTREC;

             DELETE FROM AJBOLREC
             WHERE
                (CDFILIAL    = @V_BOLREC_CDFILIAL) AND
                (NRLANBOLREC = @V_BOLREC_NRLANBOLREC);

             DELETE FROM ERRCNABBOLR
             WHERE
                (CDFILIAL    = @V_BOLREC_CDFILIAL) AND
                (NRLANBOLREC = @V_BOLREC_NRLANBOLREC);

             DELETE FROM BOLETRECEBER
             WHERE
                (CDFILIAL    = @V_BOLREC_CDFILIAL) AND
                (NRLANBOLREC = @V_BOLREC_NRLANBOLREC);
          END;
          FETCH NEXT FROM C_BOLREC INTO @V_BOLREC_CDFILIAL,     @V_BOLREC_NRLANBOLREC,
                                        @V_BOLREC_CDFILIBAIREC, @V_BOLREC_NRBAIXAREC;
      END;
      CLOSE C_BOLREC;
      DEALLOCATE C_BOLREC;
   END;

/*  EXCLUSO DAS BAIXAS PAGAS E DOS MOVIMENTOS  */

   SELECT
      @V_CTBAIXA = COUNT(*)
   FROM BAIXAPAGA
   WHERE
      (CDEMPRESA   = @P_CDEMPRESA) AND
      (DTBAIXAPAG <= @P_DTLIMITE);

   IF @V_CTBAIXA > 0 BEGIN
      DECLARE C_BAIXAPAG CURSOR FOR
      SELECT CDFILIAL, NRBAIXAPAG, CDREDUCONT, NRMOVBANC
      FROM BAIXAPAGA
      WHERE (CDEMPRESA   = @P_CDEMPRESA) AND
            (DTBAIXAPAG <= @P_DTLIMITE);
      OPEN C_BAIXAPAG;
      FETCH NEXT FROM C_BAIXAPAG INTO @V_BAIXAPAG_CDFILIAL,   @V_BAIXAPAG_NRBAIXAPAG,
                                      @V_BAIXAPAG_CDREDUCONT, @V_BAIXAPAG_NRMOVBANC;

      WHILE (@@FETCH_STATUS = 0) BEGIN
          SELECT
             @V_CTBOL = COUNT(*)
          FROM BOLETPAGAR
          WHERE
             (CDFILIBAIPAG = @V_BAIXAPAG_CDFILIAL) AND
             (NRBAIXAPAG   = @V_BAIXAPAG_NRBAIXAPAG);

          IF (@V_CTBOL = 0) BEGIN
             DELETE FROM BAIXAPAGA
             WHERE
                (CDFILIAL   = @V_BAIXAPAG_CDFILIAL) AND
                (NRBAIXAPAG = @V_BAIXAPAG_NRBAIXAPAG);

             DELETE FROM MOVBANCARIO
             WHERE
                (CDREDUCONT = @V_BAIXAPAG_CDREDUCONT) AND
                (NRMOVBANC  = @V_BAIXAPAG_NRMOVBANC);

             DELETE FROM MOVREAL
             WHERE
                (CDREDUCONT = @V_BAIXAPAG_CDREDUCONT) AND
                (NRMOVBARE  = @V_BAIXAPAG_NRMOVBANC);
          END;
          FETCH NEXT FROM C_BAIXAPAG INTO @V_BAIXAPAG_CDFILIAL,   @V_BAIXAPAG_NRBAIXAPAG,
                                          @V_BAIXAPAG_CDREDUCONT, @V_BAIXAPAG_NRMOVBANC;
      END;
      CLOSE C_BAIXAPAG;
      DEALLOCATE C_BAIXAPAG;
   END;

/*  EXCLUSO DAS BAIXAS RECEBIDAS E DOS MOVIMENTOS  */

   SELECT
      @V_CTBAIXA = COUNT(*)
   FROM BAIXARECEBIDA
   WHERE
      (CDEMPRESA   = @P_CDEMPRESA) AND
      (DTBAIXAREC <= @P_DTLIMITE);

   IF @V_CTBAIXA > 0 BEGIN
      DECLARE C_BAIXAREC CURSOR FOR
      SELECT CDFILIAL, NRBAIXAREC, CDREDUCONT, NRMOVBANC
      FROM BAIXARECEBIDA
      WHERE (CDEMPRESA   = @P_CDEMPRESA) AND
            (DTBAIXAREC <= @P_DTLIMITE);
      OPEN C_BAIXAREC;
      FETCH NEXT FROM C_BAIXAREC INTO @V_BAIXAREC_CDFILIAL,   @V_BAIXAREC_NRBAIXAREC,
                                      @V_BAIXAREC_CDREDUCONT, @V_BAIXAREC_NRMOVBANC;

      WHILE (@@FETCH_STATUS = 0) BEGIN
          SELECT
             @V_CTBOL = COUNT(*)
          FROM BOLETRECEBER
          WHERE
             (CDFILIBAIREC = @V_BAIXAREC_CDFILIAL) AND
             (NRBAIXAREC   = @V_BAIXAREC_NRBAIXAREC);

          IF (@V_CTBOL = 0) BEGIN
             DELETE FROM BAIXARECEBIDA
             WHERE
                (CDFILIAL   = @V_BAIXAREC_CDFILIAL) AND
                (NRBAIXAREC = @V_BAIXAREC_NRBAIXAREC);

             DELETE FROM MOVBANCARIO
             WHERE
                (CDREDUCONT = @V_BAIXAREC_CDREDUCONT) AND
                (NRMOVBANC  = @V_BAIXAREC_NRMOVBANC);

             DELETE FROM MOVREAL
             WHERE
                (CDREDUCONT = @V_BAIXAREC_CDREDUCONT) AND
                (NRMOVBARE  = @V_BAIXAREC_NRMOVBANC);
          END;
          FETCH NEXT FROM C_BAIXAREC INTO @V_BAIXAREC_CDFILIAL,   @V_BAIXAREC_NRBAIXAREC,
                                          @V_BAIXAREC_CDREDUCONT, @V_BAIXAREC_NRMOVBANC;
      END;
      CLOSE C_BAIXAREC;
      DEALLOCATE C_BAIXAREC;
   END;

/*  EXCLUSO DE DEBITO E CREDITO  */

   SELECT
      @V_CTDEBI = COUNT(*)
   FROM MOVDEBBANC
   WHERE
      (CDEMPRESA   = @P_CDEMPRESA) AND
      (DTMOVIDEBI <= @P_DTLIMITE);

   IF @V_CTDEBI > 0 BEGIN
      DECLARE C_MOVDEBI CURSOR FOR
      SELECT CDREDUCONT, NRMOVIDEBI, NRMOVBANC
      FROM MOVDEBBANC
      WHERE (CDEMPRESA   = @P_CDEMPRESA) AND
            (DTMOVIDEBI <= @P_DTLIMITE);
      OPEN C_MOVDEBI;
      FETCH NEXT FROM C_MOVDEBI INTO @V_MOVDEBI_CDREDUCONT, @V_MOVDEBI_NRMOVIDEBI, @V_MOVDEBI_NRMOVBANC

      WHILE (@@FETCH_STATUS = 0) BEGIN
         DELETE FROM MOVDEBBANC
         WHERE
            (CDREDUCONT = @V_MOVDEBI_CDREDUCONT) AND
            (NRMOVIDEBI = @V_MOVDEBI_NRMOVIDEBI);

         DELETE FROM MOVBANCARIO
         WHERE
            (CDREDUCONT = @V_MOVDEBI_CDREDUCONT) AND
            (NRMOVBANC  = @V_MOVDEBI_NRMOVBANC);

         DELETE FROM MOVREAL
         WHERE
            (CDREDUCONT = @V_MOVDEBI_CDREDUCONT) AND
            (NRMOVBARE  = @V_MOVDEBI_NRMOVBANC);
         FETCH NEXT FROM C_MOVDEBI INTO @V_MOVDEBI_CDREDUCONT, @V_MOVDEBI_NRMOVIDEBI, @V_MOVDEBI_NRMOVBANC
      END;
      CLOSE C_MOVDEBI;
      DEALLOCATE C_MOVDEBI;
   END;
      
   SELECT
      @V_CTCRED = COUNT(*)
   FROM MOVCREDBANC
   WHERE
      (CDEMPRESA   = @P_CDEMPRESA) AND
      (DTMOVICRED <= @P_DTLIMITE);

   IF @V_CTCRED > 0 BEGIN
      DECLARE C_MOVCRED CURSOR FOR
      SELECT CDREDUCONT, NRMOVICRED, NRMOVBANC
      FROM MOVCREDBANC
      WHERE (CDEMPRESA   = @P_CDEMPRESA) AND
            (DTMOVICRED <= @P_DTLIMITE);
      OPEN C_MOVCRED;
      FETCH NEXT FROM C_MOVCRED INTO @V_MOVCRED_CDREDUCONT, @V_MOVCRED_NRMOVICRED, @V_MOVCRED_NRMOVBANC

      WHILE (@@FETCH_STATUS = 0) BEGIN
         DELETE FROM MOVCREDBANC
         WHERE
            (CDREDUCONT = @V_MOVCRED_CDREDUCONT) AND
            (NRMOVICRED = @V_MOVCRED_NRMOVICRED);

         DELETE FROM MOVBANCARIO
         WHERE
            (CDREDUCONT = @V_MOVCRED_CDREDUCONT) AND
            (NRMOVBANC  = @V_MOVCRED_NRMOVBANC);

         DELETE FROM MOVREAL
         WHERE
            (CDREDUCONT = @V_MOVCRED_CDREDUCONT) AND
            (NRMOVBARE  = @V_MOVCRED_NRMOVBANC);
         FETCH NEXT FROM C_MOVCRED INTO @V_MOVCRED_CDREDUCONT, @V_MOVCRED_NRMOVICRED, @V_MOVCRED_NRMOVBANC
      END;
      CLOSE C_MOVCRED;
      DEALLOCATE C_MOVCRED;
   END;

/*  EXCLUSO DE TRANSFERENCIA  */

   SELECT
      @V_CTTRANSF = COUNT(*)
   FROM TRANSFBANC
   WHERE
      (CDEMPRESA   = @P_CDEMPRESA) AND
      (DTTRANBANC <= @P_DTLIMITE);

   IF @V_CTTRANSF > 0 BEGIN
      DECLARE C_TRANSF CURSOR FOR
      SELECT TF.CDEMPRESA, TF.NRTRANBANC, IT.CDREDUCONT, IT.NRMOVBANC
      FROM ITTRANSFBANC IT, TRANSFBANC   TF
      WHERE (TF.CDEMPRESA   = @P_CDEMPRESA) AND
            (TF.DTTRANBANC <= @P_DTLIMITE)  AND
            (IT.CDEMPRESA   = TF.CDEMPRESA) AND
            (IT.NRTRANBANC  = TF.NRTRANBANC);
      OPEN C_TRANSF;
      FETCH NEXT FROM C_TRANSF INTO @V_TRANSF_CDEMPRESA,  @V_TRANSF_NRTRANBANC,
                                    @V_TRANSF_CDREDUCONT, @V_TRANSF_NRMOVBANC;

      WHILE (@@FETCH_STATUS = 0) BEGIN
         DELETE FROM ITTRANSFBANC
         WHERE
            (CDEMPRESA  = @V_TRANSF_CDEMPRESA) AND
            (NRTRANBANC = @V_TRANSF_NRTRANBANC);

         DELETE FROM MOVBANCARIO
         WHERE
            (CDREDUCONT = @V_TRANSF_CDREDUCONT) AND
            (NRMOVBANC  = @V_TRANSF_NRMOVBANC);

         DELETE FROM MOVREAL
         WHERE
            (CDREDUCONT = @V_TRANSF_CDREDUCONT) AND
            (NRMOVBARE  = @V_TRANSF_NRMOVBANC);
         FETCH NEXT FROM C_TRANSF INTO @V_TRANSF_CDEMPRESA,  @V_TRANSF_NRTRANBANC,
                                       @V_TRANSF_CDREDUCONT, @V_TRANSF_NRMOVBANC;
      END;
      CLOSE C_TRANSF;
      DEALLOCATE C_TRANSF;

      DELETE FROM TRANSFBANC
      WHERE
         (CDEMPRESA   = @P_CDEMPRESA) AND
         (DTTRANBANC <= @P_DTLIMITE);
   END;
END;
GO
