/*+
|                                               |
|          Procedure: ETL_ESTOQUE               |
|                                               |
| Rotina para insero de dados de BI -         |
| valores de estoque para visualizao no CVI.  |
|                                               |
|            Tabelas Preenchidas:               |
|                                               |
|       1:   ETL_RESUESTO - Estoque             |
|                                               |
|   Criao:                                    |
|   14/08/2007 - Breno de Assis e Castro        |
|                                               |
|   Alterao:                                  |
|   19/05/2008 - Danilo dos Anjos Faria         |
|    Incluso do campo "Data de Processamento" |
+*/



IF EXISTS (SELECT *
	   FROM   SYSOBJECTS O, SYSUSERS U
           WHERE
	          O.NAME = 'ETL_ESTOQUE'
           AND    TYPE   = 'P'
	   AND    U.UID  = O.UID
	   AND    U.NAME = USER
           )
	  DROP PROCEDURE ETL_ESTOQUE
GO
CREATE PROCEDURE ETL_ESTOQUE
   (@P_DTINI       DATETIME,
    @P_DTFINA      DATETIME )
AS
   DECLARE   @V_DTFIM                  DATETIME;
   DECLARE   @V_DTINI                  DATETIME;

   DECLARE   @V_ESTOQUE_INICIAL        DOUBLE PRECISION;
   DECLARE   @V_ESTOQUE_FINAL          DOUBLE PRECISION;

   DECLARE   @V_DIAS_ESTOQUE           DOUBLE PRECISION;
   DECLARE   @V_QTINVENTARIADA         DOUBLE PRECISION;
   DECLARE   @V_QTESTOQUE              DOUBLE PRECISION;
   DECLARE   @V_VRESTOQUE              DOUBLE PRECISION;

   DECLARE   @V_DIA                    VARCHAR(02);
   DECLARE   @V_MES                    VARCHAR(02);
   DECLARE   @V_ANO                    VARCHAR(04);
   DECLARE   @V_BIM                    VARCHAR(06);
   DECLARE   @V_TRI                    VARCHAR(06);
   DECLARE   @V_SEM                    VARCHAR(06);

   DECLARE   @V_ZERO                   DOUBLE PRECISION;
   DECLARE   @V_CONT                   DOUBLE PRECISION;

/*DECLARE VARIAVEIS IN C_ETL*/
   DECLARE   @V_CDFILIAL_ETL           VARCHAR(04);
   DECLARE   @V_CDAREAGERE 	         VARCHAR(03);
   DECLARE   @V_CDSEGMER 	            VARCHAR(05);
   DECLARE   @V_CDEMPRESA 	            VARCHAR(02);
   DECLARE   @V_CDSUPERVISOR 	         VARCHAR(12);
   DECLARE   @V_DTINICIALBI 	         DATETIME;
   DECLARE   @V_CDGOPERAC 	            VARCHAR(12);
   DECLARE   @V_DTPROCESSA             DATETIME;

/*DECLARE VARIAVEIS IN C_ESTOQUE*/
   DECLARE   @V_QTESTDIACON            DOUBLE PRECISION;
   DECLARE   @V_CDFILIALEST            VARCHAR(04);
   DECLARE   @V_VRESTDIACON            DOUBLE PRECISION;
   DECLARE   @V_CDPRODUTO              VARCHAR(10);
   DECLARE   @V_NMPRODUTO              VARCHAR(50);

   DECLARE   @V_PRXMOVI                VARCHAR(20);
   DECLARE   @V_PRODINI                VARCHAR(20);
   DECLARE   @V_PRODFIN                VARCHAR(20);
   DECLARE   @V_NIVEL                  INTEGER;
   DECLARE   @V_QTDEMOVIMENTADA        DOUBLE PRECISION;
   DECLARE   @V_QTDEDIASNAOUTEIS       DOUBLE PRECISION;
   DECLARE   @V_RETIRADAMEDIADIA       DOUBLE PRECISION;

   DECLARE   @V_ETL_CUSTO              INTEGER;
   DECLARE   @V_ETL_IDUTILALMOX        VARCHAR(1);
   DECLARE   @V_ETL_CDALMOXARIFE       VARCHAR(6);

   DECLARE   @V_DIFERENCA_DIAS         DOUBLE PRECISION;
BEGIN

   SET  @V_VRESTOQUE 	   = 0;
   SET  @V_ESTOQUE_INICIAL = 0;
   SET  @V_ESTOQUE_FINAL   = 0;
   SET  @V_ZERO            = 0;
   SET  @V_CONT            = 0;
   SET  @V_QTESTOQUE       = 0;


   SET @V_MES =  (CONVERT(VARCHAR, MONTH(@P_DTINI)));
   SET @V_MES = RIGHT(REPLICATE('0',2)+ @V_MES, 2);
   SET @V_ANO =  (CONVERT(VARCHAR, YEAR (@P_DTINI)));

   BEGIN TRANSACTION
      DELETE FROM  ETL_RESUESTO
      WHERE  (CDANOBI > @V_ANO )
      OR     (CDANOBI = @V_ANO AND CDMESBI >= @V_MES );
   COMMIT TRANSACTION;

   SET @V_DTINI = CONVERT(DATETIME,('01/'+@V_MES+'/'+@V_ANO),103);

   SELECT @V_NIVEL = LEN(MAX(CDARVPROD)), @V_PRODINI = MIN(CDARVPROD), @V_PRODFIN = MAX(CDARVPROD)
     FROM PRODUTO
    WHERE PRODUTO.CDPRODESTO IS NOT NULL;

   /*INICIO WHILE LOOP VERIFICA DATA INICIAL / DATA FINAL*/
   WHILE (@V_DTINI <= @P_DTFINA)BEGIN
      BEGIN TRANSACTION/*INICIO BEGIN PARAR COMMIT*/
         /*INICIO WHILE CURSOR C_ETL */
      SET @V_DTFIM = DATEADD(MM,1,@V_DTINI)-DAY(@V_DTINI);
      DECLARE C_ETL CURSOR  FOR
         (SELECT F.CDFILIAL,     F.CDAREAGERE,      F.CDSEGMER,
                 F.CDEMPRESA,    F.CDSUPERVISOR,    F.DTINICIALBI,
                 F.CDGOPERAC,    P.DTPROCESSA, CONVERT(INTEGER, ISNULL(P.CDTIPCUSTEST, 3)) CUSTO,
                 P.IDUTILALMOX,  P.CDALMOXARIFE
          FROM   FILIAL F, PARAMFILIAL P
          WHERE  P.CDFILIAL = F.CDFILIAL
            AND  F.IDFUNCAOFILI IS NULL
            AND  F.DTINICIALBI IS NOT NULL
         );
      OPEN C_ETL;
      FETCH NEXT FROM C_ETL INTO
                           @V_CDFILIAL_ETL, @V_CDAREAGERE,     @V_CDSEGMER,
                           @V_CDEMPRESA,    @V_CDSUPERVISOR,   @V_DTINICIALBI,
                           @V_CDGOPERAC,    @V_DTPROCESSA, @V_ETL_CUSTO,
                           @V_ETL_IDUTILALMOX, @V_ETL_CDALMOXARIFE;
      WHILE (@@FETCH_STATUS = 0) BEGIN
         IF CONVERT (DATETIME, ('01' + '/' + RIGHT(REPLICATE('0',2)+ CONVERT(VARCHAR, MONTH(@V_DTINICIALBI)), 2) +'/'+ CONVERT (VARCHAR, YEAR(@V_DTINICIALBI)))) <= @V_DTINI BEGIN
            SET @V_QTDEDIASNAOUTEIS=0;
            SELECT @V_QTDEDIASNAOUTEIS = COUNT(1)
             FROM FERIFILI
            WHERE CDFILIAL = @V_CDFILIAL_ETL
              AND DTFERIFILI BETWEEN @V_DTINI AND @V_DTFIM;

            EXEC NOVO_CODIGO 'MOVIESTOQUE', @P_SEQUENCIAL = @V_PRXMOVI OUTPUT, @P_QTDE = 1;
            SET @V_PRXMOVI = SUBSTRING(@V_PRXMOVI,11,10);

            IF @V_ETL_IDUTILALMOX = 'S' BEGIN
               EXEC TEKNISA.MOVIMENTACAO_ESTOQUE @V_CDFILIAL_ETL, @V_ETL_CDALMOXARIFE,'','', @V_NIVEL, @V_DTINI, @V_DTFIM, @V_PRODINI, @V_PRODFIN, 3, @V_ETL_CUSTO, @V_PRXMOVI;
            END
            ELSE BEGIN
               EXEC TEKNISA.MOVIMENTACAO_ESTOQUE @V_CDFILIAL_ETL,' ',' ',' ', @V_NIVEL, @V_DTINI, @V_DTFIM, @V_PRODINI, @V_PRODFIN, 3, @V_ETL_CUSTO, @V_PRXMOVI;
            END;

            /*INICIO WHILE CURSOR C_ESTOQUE / VERIFICAO DATA INICIL BI*/
            DECLARE C_ESTOQUE CURSOR FOR
               (SELECT B.QTESTDIACON,      B.VRESTDIACON,
                       B.CDFILIAL,         B.CDPRODUTO,      P.NMPRODUTO
                FROM   POSESTCONS B, PRODUTO P
                WHERE  B.CDFILIAL     =   @V_CDFILIAL_ETL
                  AND  B.CDPRODUTO    =    P.CDPRODUTO
                  AND  B.QTESTDIACON <> 0
                  AND (B.DTPOSESTCON  =  (SELECT  MAX(P.DTPOSESTCON)
                                          FROM    POSESTCONS P
                                          WHERE   (P.CDFILIAL  = B.CDFILIAL  )
                                            AND   (P.CDPRODUTO = B.CDPRODUTO )
                                            AND   (CONVERT(DATETIME, P.DTPOSESTCON, 103) <= (@V_DTFIM + 1))
                                          )
                       )
                );
            OPEN C_ESTOQUE;
            FETCH NEXT FROM C_ESTOQUE INTO
                                     @V_QTESTDIACON ,     @V_VRESTDIACON,
                                     @V_CDFILIALEST,      @V_CDPRODUTO,
                                     @V_NMPRODUTO;
            WHILE (@@FETCH_STATUS = 0) BEGIN
               /* Inventario Anterior - INVENTARIO DO ESTOQUE INICIAL */
               DECLARE  C_INVENTANT CURSOR FOR
                  (SELECT SUM (P.QTESTDIACON * TEKNISA.CUSTO_PRODUTO (@V_CDFILIAL_ETL, @V_CDPRODUTO, @V_ETL_CUSTO, ((@V_DTINI) - 1), GETDATE()))
                   FROM  POSESTCONS P
                   WHERE (P.QTESTDIACON <> 0                  )
                     AND (P.CDPRODUTO    =   @V_CDPRODUTO     )
                     AND (P.CDFILIAL     =   @V_CDFILIAL_ETL  )
                     AND (P.DTPOSESTCON  =   (SELECT MAX( C.DTPOSESTCON )
                                              FROM   POSESTCONS C
                                              WHERE  ( C.CDFILIAL     = P.CDFILIAL  )
                                                AND  ( C.CDPRODUTO    = P.CDPRODUTO )
                                                AND  (CONVERT(DATETIME, P.DTPOSESTCON, 103) <= (@V_DTINI + 1))
                                             )
                          )
                   );
               OPEN C_INVENTANT;
               FETCH NEXT FROM C_INVENTANT INTO @V_ESTOQUE_INICIAL;
               WHILE (@@FETCH_STATUS = 0) BEGIN
                  /* Inventario FINAL - INVERNTARIO DO ESTOQUE FINAL*/
                  DECLARE C_INVENTFIN CURSOR FOR
                     (SELECT SUM(P.QTESTDIACON * TEKNISA.CUSTO_PRODUTO(@V_CDFILIAL_ETL, @V_CDPRODUTO, @V_ETL_CUSTO, @V_DTFIM, GETDATE()))
                      FROM   POSESTCONS P
                      WHERE  (P.QTESTDIACON  <> 0                )
                        AND  (P.CDPRODUTO    =   @V_CDPRODUTO    )
                        AND  (P.CDFILIAL     =   @V_CDFILIAL_ETL )
                        AND  (P.DTPOSESTCON  =   (SELECT MAX( C.DTPOSESTCON )
                                                  FROM   POSESTCONS C
                                                  WHERE  (C.CDFILIAL      =  P.CDFILIAL  )
                                                    AND  (C.CDPRODUTO     =  P.CDPRODUTO )
                                                    AND  (CONVERT(DATETIME, P.DTPOSESTCON, 103) <= (@V_DTFIM + 1))
                                                  )
                              )
                     );
                  OPEN C_INVENTFIN;
                  FETCH NEXT FROM C_INVENTFIN INTO @V_ESTOQUE_FINAL;
                  WHILE (@@FETCH_STATUS = 0) BEGIN

                     /*CALCULOS DIAS ESTOQUE / QUANTIDADE INVENTARIADA*/
  --================================================================================================
                     /* Calcula dias de ESTOQUE */

                     SELECT @V_QTDEMOVIMENTADA = SUM(QTLANCAMENTO)
                      FROM RELAT_MOVIEST
                      WHERE NRRELATMOVI = @V_PRXMOVI
                        AND CDPRODMOVIEST = @V_CDPRODUTO
                        AND CDFILIMOVIEST = @V_CDFILIAL_ETL
                        AND IDTIPOMOVIEST = '3';


                     SET @V_DIFERENCA_DIAS = CONVERT(DOUBLE PRECISION, @V_DTFIM) - CONVERT(DOUBLE PRECISION, @V_DTINI);

                     IF (@V_DIFERENCA_DIAS - @V_QTDEDIASNAOUTEIS) > 0 BEGIN
                        SET @V_RETIRADAMEDIADIA = @V_QTDEMOVIMENTADA/(@V_DIFERENCA_DIAS - @V_QTDEDIASNAOUTEIS);
                     END
                     ELSE BEGIN
                        SET @V_RETIRADAMEDIADIA = @V_QTDEMOVIMENTADA/30;
                     END;

                     SET @V_RETIRADAMEDIADIA = (@V_RETIRADAMEDIADIA * TEKNISA.CUSTO_PRODUTO(@V_CDFILIAL_ETL, @V_CDPRODUTO, @V_ETL_CUSTO, @V_DTFIM, GETDATE()));

                     IF (@V_ESTOQUE_FINAL > 0) AND (@V_RETIRADAMEDIADIA > 0) BEGIN
                        SET @V_DIAS_ESTOQUE = ROUND(@V_ESTOQUE_FINAL/@V_RETIRADAMEDIADIA, 0);
                     END
                     ELSE BEGIN
                        SET @V_DIAS_ESTOQUE = 0;
                     END;


                     /* Fim do if calcula dias ESTOQUE  */

                     /* Calcula quantidade inventariada */
                     SET @V_QTINVENTARIADA = ((@V_ESTOQUE_FINAL) - (@V_ESTOQUE_INICIAL));
  --================================================================================================

                     SET @V_MES  =  CONVERT (VARCHAR, MONTH(@V_DTFIM));
                     SET @V_MES = RIGHT(REPLICATE('0',2)+ @V_MES, 2);
                     SET @V_ANO  =  CONVERT (VARCHAR, YEAR(@V_DTFIM));
                     SET @V_BIM  =  CONVERT (VARCHAR, (ROUND(((CONVERT(DOUBLE PRECISION, @V_MES) - 1) / 2),0) + 1))+' Bim';
                     SET @V_TRI  =  CONVERT (VARCHAR, (ROUND(((CONVERT(DOUBLE PRECISION, @V_MES) - 1) / 3),0) + 1))+' Tri';
                     SET @V_SEM  =  CONVERT (VARCHAR, (ROUND(((CONVERT(DOUBLE PRECISION, @V_MES) - 1) / 6),0) + 1))+' Sem';

                     INSERT INTO ETL_RESUESTO
                                     ( CDSUPERVISOR,       CDGOPERAC,        CDEMPRESA,        CDFILIAL,
                                       CDAREAGERE,         CDSEGMER,         CDPRODUTO,        CDANOBI,
                                       CDMESBI,            BIETLEST,         TRETLEST,         SEETLEST,
                                       QTESTOETLEST,       QTINVENTETLEST,   QTDIAESTETLEST,   VRTOTESTETLEST,
                                       DSDTPROCETLESTO,    VRUNIESTETLEST
                                     )
                     VALUES
                                     ( @V_CDSUPERVISOR,    @V_CDGOPERAC,     @V_CDEMPRESA,     @V_CDFILIAL_ETL,
                                       @V_CDAREAGERE,      @V_CDSEGMER,      @V_CDPRODUTO,     @V_ANO,
                                       @V_MES,             @V_BIM,           @V_TRI,           @V_SEM,
                                       @V_QTESTDIACON,     @V_QTINVENTARIADA,@V_DIAS_ESTOQUE,  @V_VRESTDIACON,
                                       CONVERT(CHAR(10), @V_DTPROCESSA,103),
                                       TEKNISA.CUSTO_PRODUTO(@V_CDFILIAL_ETL, @V_CDPRODUTO, @V_ETL_CUSTO, @V_DTFIM, GETDATE())
                                      );

                     SET @V_CONT = @V_CONT + 1;
                     IF (@V_CONT >= 500) BEGIN
                        COMMIT TRANSACTION;
                        SET @V_CONT = 0;
                        BEGIN TRANSACTION;
                     END; /* Fim IF COMMIT para 500 */

                     FETCH NEXT FROM C_INVENTFIN INTO @V_ESTOQUE_FINAL;
                  END; /*FIM WHILE CURSOR C_INVENTFIN*/
                  CLOSE C_INVENTFIN;
                  DEALLOCATE C_INVENTFIN;
                  FETCH NEXT FROM C_INVENTANT INTO @V_ESTOQUE_INICIAL;
               END; /*FIM WHILE  CURSOR C_INVENTANT*/
               CLOSE C_INVENTANT;
               DEALLOCATE C_INVENTANT;
               FETCH NEXT FROM C_ESTOQUE INTO
                                        @V_QTESTDIACON ,     @V_VRESTDIACON,
                                        @V_CDFILIALEST,      @V_CDPRODUTO,
                                        @V_NMPRODUTO ;
            END; /*FIM WHILE CURSOR C_ESTOQUE*/
            CLOSE C_ESTOQUE;
            DEALLOCATE C_ESTOQUE;
         END; /*FIM CONDIO DATA INICIAL BI*/
         FETCH NEXT FROM C_ETL INTO
                              @V_CDFILIAL_ETL, @V_CDAREAGERE,     @V_CDSEGMER,
                              @V_CDEMPRESA,    @V_CDSUPERVISOR,   @V_DTINICIALBI,
                              @V_CDGOPERAC,    @V_DTPROCESSA, @V_ETL_CUSTO,
                              @V_ETL_IDUTILALMOX, @V_ETL_CDALMOXARIFE;
      END; /*FIM CURSOR C_ETL */
      CLOSE C_ETL;
      DEALLOCATE C_ETL;
      COMMIT TRANSACTION;
      SET @V_DTINI = DATEADD(MM, 1, @V_DTINI);
   END; /* FIM LOOP WHILE DATA INICIAL / DATA FIM */
END;
GO

BEGIN TRAN;
   -- Cria as dimenses para a tabela ETL_RESUESTO (no cria para data de processamento)
   EXEC CRIA_DIME 'ETL_RESUESTO', 'CDSUPERVISOR'    , 'Supervisor'             , 'SUPERVISOR'  , 'CDSUPERVISOR'  , 'NMSUPERVISOR', NULL ;
   EXEC CRIA_DIME 'ETL_RESUESTO', 'CDGOPERAC'       , 'Gerente Operacional'    , 'GOPERAC'     , 'CDGOPERAC'     , 'NMGOPERAC'   , NULL ;
   EXEC CRIA_DIME 'ETL_RESUESTO', 'CDEMPRESA'       , 'Empresa'                , 'EMPRESA'     , 'CDEMPRESA'     , 'NMEMPRESA'   , NULL ;
   EXEC CRIA_DIME 'ETL_RESUESTO', 'CDFILIAL'        , 'Filial'                 , 'FILIAL'      , 'CDFILIAL'      , 'NMFILIAL'    , NULL ;
   EXEC CRIA_DIME 'ETL_RESUESTO', 'CDSEGMER'        , 'Segmento de Mercado'    , 'SEGMERCADO'  , 'CDSEGMER'      , 'DSSEGMER'    , NULL ;
   EXEC CRIA_DIME 'ETL_RESUESTO', 'CDAREAGERE'      , 'Regional'               , 'AREAGERE'    , 'CDAREAGERE'    , 'NMAREAGERE'  , NULL ;
   EXEC CRIA_DIME 'ETL_RESUESTO', 'CDPRODUTO'       , 'Produto'                , 'PRODUTO'     , 'CDPRODUTO'     , 'NMPRODUTO'   , NULL ;
   EXEC CRIA_DIME 'ETL_RESUESTO', 'CDANOBI'         , 'Ano'                    , 'ETLANOBI'    , 'CDANOBI'       , 'NMMESBI'     , NULL ;
   EXEC CRIA_DIME 'ETL_RESUESTO', 'CDMESBI'         , 'Ms'                    , 'ETLMESBI'    , 'CDMESBI'       , 'NMMESBI'     , NULL ;
   EXEC CRIA_DIME 'ETL_RESUESTO', 'BIETLEST'        , 'Bimestre'               , NULL          , NULL            , NULL          , NULL ;
   EXEC CRIA_DIME 'ETL_RESUESTO', 'TRETLEST'        , 'Trimestre'              , NULL          , NULL            , NULL          , NULL ;
   EXEC CRIA_DIME 'ETL_RESUESTO', 'SEETLEST'        , 'Semestre'               , NULL          , NULL            , NULL          , NULL ;

   COMMIT TRAN;
GO

-- VERSAO --
EXEC ATUVEROBJ 'ETL_ESTOQUE','5.05.105';
GO




