
// FILE: src/repositories/InventarioRotativo_Repository.js
var InventarioRotativo_Repository = function (restEngine) {

    this.saveInverota = function(rows) {
        return restEngine.post('peblib#/crud/inverota/save', {
            "requestType": "DataSet",
            "dataset": rows
        });
    };

    this.saveInverotalote = function(rows) {
        return restEngine.post('peblib#/crud/inverotalote/save', {
            "requestType": "DataSet",
            "dataset": rows
        });
    };

    this.atualizaInverota = function(filter){
        return restEngine.post('peblib#/atualiza_inverota', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    /*
     *  Envia uma requisição vazia para somente verificar se cairá na
     *  exception do preDispatch de inventário rotativo no backend.
     */
    this.checkInventarioPendente = function() {
        return restEngine.post('peblib#/verifica_inventario_pendente', {
            "requestType": "Row",
            "row": {}
        });
    };
};

Configuration(function(ContextRegister){
    ContextRegister.register('InventarioRotativo_Repository', InventarioRotativo_Repository);
});


// FILE: src/repositories/Lib00206_InfComplProduto_Repository.js
var Lib00206_InfComplProduto_Repository = function(restEngine) {
    this.getProximoLote = function(filter){
        return restEngine.post('peblib#/prox_lote', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.getProdFili= function(filter){
        return restEngine.post('peblib#/crud/prodfili', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.saveLote= function(row){
        return restEngine.post('peblib#/save_lote', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00206_InfComplProduto_Repository', Lib00206_InfComplProduto_Repository);
});

// FILE: src/repositories/ValidacoesEstoque_Repository.js
var ValidacoesEstoque_Repository = function (localStorageEngine, restEngine) {
    var row = {};

    this.validaDataLancamento = function (filial, dtLancamento, operacao, sistema, dtEmissao) {

        row.CDFILIAL = filial;
        row.DTLANCTOEST = dtLancamento;
        row.OPERACAO = operacao;
        row.IDSISTEMA = sistema;
        row.DTEMISSAO = dtEmissao;

        return restEngine.post('peblib#/validacoes_estoque/data_lancamento', {
            "requestType": "Row",
            "row": row
        });
    };

    /* **************************************************
     * Função que requisita a execução da atualização de
     * estoque. Para maiores informações, favor verificar
     * a função de mesmo nome no Service de validações de
     * estoque.
     *
     * @param string cdfilial
     * @param string nrlancestq
     * @param boolean verifyIrregularities
     * @param boolean abortAtIrregularities
     * @param Array seqItemCodes
     *
     * @return Promise
     */
    this.atualizaEstoque = function (cdfilial, nrlancestq, verifyIrregularities, abortAtIrregularities, seqItemCodes, boolConsolidateToStock) {
        //sets row values
        row.CDFILIAL = cdfilial;
        row.NRLANCESTQ = nrlancestq;
        row.IDVERIRREG = verifyIrregularities;
        row.IDABRIRREG = abortAtIrregularities;
        row.SEQITEMS = seqItemCodes;
		row.CONSOLIDATE = boolConsolidateToStock;

        //sends a request to the backend
        return restEngine.post('peblib#/validacoes_estoque/atualiza_estoque', {
            "requestType": "Row",
            "row": row
        });
    };

    this.zeraInventario = function (filial, nrLancamento, idordemclc, boolConsolidateToStock) {

        row.CDFILIAL = filial;
        row.NRLANCAMENTO = nrLancamento;
        row.IDORDEMCLC = idordemclc;
		row.CONSOLIDATE = boolConsolidateToStock;

        return restEngine.post('peblib#/validacoes_estoque/zera_inventario', {
            "requestType": "Row",
            "row": row
        });
    };

    this.consultaCustoQuantidadeProduto = function (
        filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote
    )
    {
        row.CDFILIAL = filial;
        row.CDTIPOCUST = tipoCusto;
        row.CDPRODUTO = produto;
        row.CDALMOXARIFE = almoxarifado;
        row.CDLOCALESTOQ = localEstoque;
        row.NRLOTE = lote;
        row.NRSUBLOTE = subLote;
        row.DTLANCAMENTO = dtLancamento;

        return restEngine.post('peblib#/validacoes_estoque/consulta_custo', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.getPosicaoDeEstoquePorProduto = function(filter) {
        return restEngine.post('peblib#/retirada_posestdia', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ValidacoesEstoque_Repository', ValidacoesEstoque_Repository);
});

// FILE: src/services/InventarioRotativo_Service.js
var InventarioRotativo_Service = function(localStorageEngine, restEngine, InventarioRotativo_Repository, ZHPromise) {

	this.verificaInvePendente = function(cdfilial) {
        var filter = [{
            "name": "P_CDFILIAL",
            "operator": "=",
            "value": cdfilial
        }];
        return localStorageEngine.download('/verificaInvePend',filter);
    };

    this.save = function(rowsToSave, utLote) {
    	if (utLote) {
    	    return InventarioRotativo_Repository.saveInverotalote(rowsToSave);
        }
    	else {
    	    return InventarioRotativo_Repository.saveInverota(rowsToSave);
        }
    };

    this.getOperadorLogado = function() {
         var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];

        return localStorageEngine.download("peblib#/crud/operador", filter);
    };

    this.atualizaInventarioRotativo = function(filter) {
        return InventarioRotativo_Repository.atualizaInverota(filter);
    };

    /*
     *  Função para verificar se há inventário pendente.
     *  Como há um preDispatch no backend, o retorno cairá num callback
     *  de erro se houver inventário rotativo pendente, devido a uma
     *  exception que é lançada. Se não houver inventário rotativo, o retorno
     *  será no callback de sucesso.
     */
    this.checkInventarioPendente = function() {
        var deferred = ZHPromise.defer();
        InventarioRotativo_Repository.checkInventarioPendente().then(function(response) {
            // Verifica se realmente entrou na função de verificação e
            // retornou o dataset "inventario_valido".
            if (response.dataset && response.dataset.inventario_valido) {
                deferred.resolve(true);
            }
            else {
                deferred.reject(response);
            }
        }, function(error) {
            // Verifica se o erro tratado é a exceção de inventário rotativo pendente
            if (error && Util.isObject(error.data) && error.data.error === "Exception: Inventário Rotativo Pendente") {
                deferred.resolve(false);
            }
            else {
                deferred.reject(error);
            }
        });
        return deferred;
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('InventarioRotativo_Service', InventarioRotativo_Service);
});

// FILE: src/services/Lib00206_InfComplProduto_Service.js
var Lib00206_InfComplProduto_Service = function(Lib00206_InfComplProduto_Repository, restEngine) {
    this.getDtprocessa = function(){
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return restEngine.post("peblib#/crud/paramestoque_processamento", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.getParamFilial = function(cdFiliMovi){
        var filial = cdFiliMovi || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            }
        ];
        return restEngine.post("peblib#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.getInfoLote = function(cdFiliMovi,cdProdMovi,nrLoteEstoq,nrSubLote){
        return this.getDtprocessa();
    };

    this.saveLote = function(row){
        row.NRLOTEESTQ = row.NRLOTEESTQ || ' ';
        row.NRSUBLOTE  = row.NRSUBLOTE  || ' ';
        row.DTVALILOTE = row.DTVALILOTE || null;
        row.DTFABRPROD = row.DTFABRPROD || null;

        var saveRow = {
            CDFILIMOVI:row.CDFILIMOVI,
            NRLOTEESTQ:row.NRLOTEESTQ,
            NRSUBLOTE:row.NRSUBLOTE,
            CDPRODESTO:row.CDPRODESTO,
            DTVALILOTE:row.DTVALILOTE,
            DTFABRPROD:row.DTFABRPROD,
            VRFATOCONV: row.VRFATOCONV
        };

        return Lib00206_InfComplProduto_Repository.saveLote(saveRow);
    };

    this.getProdFili = function(cdproduto,cdfilial){
        var filter = [{
            'name':"CDPRODUTO",
            "operator":"=",
            "value":cdproduto
        },
        {
            'name':"CDFILIAL",
            "operator":"=",
            "value":cdfilial
        }];
        return Lib00206_InfComplProduto_Repository.getProdFili(filter);
    };

    this.getProximoLote = function(cdFilial,cdProdesto){
        var filter = [
            {
                "name":"CDFILIAL",
                "operator":"=",
                "value":cdFilial
            },
            {
                "name":"DTVALILTE",
                "operator":"=",
                "value":moment().format('DD/MM/YYYY')
            },
            {
                "name":"CDPRODUTO",
                "operator":"=",
                "value":cdProdesto
            }
        ];
        return Lib00206_InfComplProduto_Repository.getProximoLote(filter);
    };

	this.validateInventory = function(widget, params){
	    
	    if(!params)
	        params = {};
	        
        var filial = widget.currentRow.CDFILIMOVI;
        
        var almoxarifado = widget.getField('CDALMOXARIFE');
        var local = widget.getField('CDLOCALESTOQ');
        var lote = widget.getField('NRLOTEESTQ');
        var dataSourceName = 'inverota';
        if(params != {})
            widget.params = params;
        
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            },
            {
                "name": "CDALMOXARIFE",
                "operator": "=",
                "value": almoxarifado.value() || ' '
            },
            {
                "name": "CDLOCALESTOQ",
                "operator": "=",
                "value": local.value() || ' '
            },
        ];
        
        if(widget.params.IDUTILLOTE != 'S'){
            dataSourceName = 'inverota';
            filter.push({
                "name": "IDINVENTOK",
                "operator": "=",
                "value": "N"
            });
        }else{
            dataSourceName = 'inverotalote';
            filter.push({
                "name": "IDINVLOTEOK",
                "operator": "=",
                "value": "N"
            });
        }
        
        return restEngine.post('peblib#/crud/'+dataSourceName, {
            "requestType": "FilterData",
            "filter": filter
        }).then(function(resolve) {
            return resolve.dataset[dataSourceName];
        }.bind(this));
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00206_InfComplProduto_Service', Lib00206_InfComplProduto_Service);
});

// FILE: src/services/Lib00207_PosicaoEstoque_Service.js
var Lib00207_PosicaoEstoque_Service = function(General) {
    
    this.getDtprocessa = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("peblib#/crud/paramestoque_processamento", {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.getParamFilial = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("peblib#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.getPosicaoDeEstoquePorProduto = function(row) {
	    var filter = [
	    	{
	    		"name": "CDPRODUTO",
				"operator": "=",
				"value": row.CDPRODUTO
			},
			{
	    		"name": "CDFILIAL",
				"operator": "=",
				"value": row.CDFILIAL
			},
			{
	    		"name": "CDALMOXARIFE",
				"operator": "=",
				"value": row.CDALMOXARIFE
			},
			{
	    		"name": "CDLOCALESTOQ",
				"operator": "=",
				"value": row.CDLOCALESTOQ
			},
			{
	    		"name": "NRLOTEESTQ",
				"operator": "=",
				"value": row.NRLOTEESTQ
			},
			{
	    		"name": "NRSUBLOTE",
				"operator": "=",
				"value": row.NRSUBLOTE
			}
		];

        return General.callRestEngine('peblib#/retirada_posestdia', 'FilterData', filter).then(function(success){
            return success.dataset.posestdia[0];
        });
	
	};
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00207_PosicaoEstoque_Service', Lib00207_PosicaoEstoque_Service);
});

// FILE: src/services/ValidacoesEstoque_Service.js
var ValidacoesEstoque_Service = function(ValidacoesEstoque_Repository, ScreenService, metaDataFactory) {

    var TIPO_OPERACAO = {
        'ENTRADA':    'E',
        'SAIDA':      'S',
        'AJUSTE_IMP': 'A'
    };

    var TIPO_SISTEMA = [
        'DFE', 'DFS', 'EST', 'FAI', 'FAZ', 'FAC', 'REC', 'PAG'
    ];

    var TIPO_CUSTO_PADRAO = '03',
        VAZIO_EST         = ' ';

    this.validaDataLancamento = function(filial, dataLancamento, operacao, sistema, dataEmissao) {
        var validaOperacaoSistema = (!TIPO_OPERACAO.hasOwnProperty(operacao)) || TIPO_SISTEMA.indexOf(sistema) === -1;

        if(!dataLancamento || !operacao || validaOperacaoSistema) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        operacao = TIPO_OPERACAO[operacao];
        dataLancamento = moment(dataLancamento, "DD/MM/YYYY").format('DD/MM/YYYY');
        dataEmissao = !dataEmissao ? dataLancamento : moment(dataEmissao, "DD/MM/YYYY").format('DD/MM/YYYY');

        return ValidacoesEstoque_Repository.validaDataLancamento(filial, dataLancamento, operacao, sistema, dataEmissao);
    };

    /* **************************************************
     * Função que requisita a execução da atualização de
     * estoque. Possui como comportamento padrão a
     * atualização de todos os itens de um lançamento de
     * uma unidade, com verificação de irregularidades e
     * abortamento das operações no caso de invalidez
     *
     * Permite porém que a verificação de irregularidades
     * não seja efetuada ou que as operações não sejam
     * abortadas após a verificação através da passagem
     * dos parâmetros "verifyIrregularities" e
     * "abortAtIrregularities". Note que o último não
     * pode ser verdadeiro se o primeiro for falso.
     *
     * Permite também que sejam selecionados um ou mais
     * itens para a execução da atualização de estoque
     * através da passagem de seus códigos na variável
     * "seqItemCodes". Ela é nula no comportamento
     * padrão, o que garante que todos os itens do
     * lançamento sejam atualizados.
     *
     * @param string  cdfilial
     * @param string  nrlancestq
     * @param boolean verifyIrregularities
     * @param boolean abortAtIrregularities
     * @param Array   seqItemCodes
     *
     * @return Promise
     */
    this.atualizaEstoque = function(cdfilial, nrlancestq, boolConsolidateToStock, verifyIrregularities, abortAtIrregularities, seqItemCodes) {
        //ensures that "seqItemCodes" will never be "undefined", as such value doesn't exist in PHP
        seqItemCodes = seqItemCodes || null;

        //sets default flag values if and only if they haven't been passed
        if(typeof verifyIrregularities === 'undefined')
            verifyIrregularities = true;
        if(typeof abortAtIrregularities === 'undefined')
            abortAtIrregularities = verifyIrregularities;
		if(typeof boolConsolidateToStock === 'undefined')
            boolConsolidateToStock = false;

        //checks for invalid parameter values and returns rejected promise
        if(!cdfilial || !nrlancestq ||
           (typeof verifyIrregularities !== 'boolean') ||
           (typeof abortAtIrregularities !== 'boolean') ||
		   (typeof boolConsolidateToStock !== 'boolean') ||
           (!verifyIrregularities && abortAtIrregularities)) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';

            return Promise.reject(response);
        }

        //calls repository's request function, returning its promise
        return ValidacoesEstoque_Repository.atualizaEstoque(
            cdfilial, nrlancestq, verifyIrregularities,
            abortAtIrregularities, seqItemCodes, boolConsolidateToStock
        );
    };

    this.zeraInventario = function(filial, nrLancamento, idordemclc, boolConsolidateToStock) {
        if(typeof boolConsolidateToStock === 'undefined')
            boolConsolidateToStock = false;
            
        idordemclc = (idordemclc !== 'U')? 0 :'U';

        if(!filial || !nrLancamento) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        return ValidacoesEstoque_Repository.zeraInventario(filial, nrLancamento, idordemclc, boolConsolidateToStock);
    };

    this.consultaCustoQuantidadeProduto = function(filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote) {
        if(!filial || !produto || !dtLancamento) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        almoxarifado = almoxarifado || VAZIO_EST;
        localEstoque = localEstoque || VAZIO_EST;

        lote    =    lote || VAZIO_EST;
        subLote = subLote || VAZIO_EST;

        tipoCusto = tipoCusto || TIPO_CUSTO_PADRAO;

        return ValidacoesEstoque_Repository.consultaCustoQuantidadeProduto(
            filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote
        );
    };
    
    this.openPosicaoDeEstoque = function(widget) {
    	var popup = this.getPosicaoDeEstoquePopup(widget);

		ScreenService.openPopup(popup);

		popup.currentRow = widget.currentRow;
		popup.parentWidget = widget;

		this.getDataForPosicaoDeEstoquePopup(popup);
    };
    
    this.getDataForPosicaoDeEstoquePopup = function(popup) {
    	this.getPosicaoDeEstoquePorProduto(popup.currentRow).then(function(posestdia) {
    		if(posestdia) {
	    		popup.currentRow = posestdia;
	    		popup.parentWidget.currentRow.QTESTOQDIA = posestdia.QTESTOQDIA;
    		}
    		else{
    			popup.currentRow = 0;
    		}
    	}).catch(function(error) {
    		ScreenService.showMessage(error.data.error);
    	});
    };
    
    this.getPosicaoDeEstoquePopup = function(parentWidget) {
        var widget = parentWidget.posestPopup;

        if(!widget) {
            widget = this.getPosicaoDeEstoquePopUpObject();
            widget.container = parentWidget.container;
            widget.label = widget.defaultLabel + parentWidget.currentRow.NMPRODUTO;

            widget = metaDataFactory.widgetFactory(widget, parentWidget);

            parentWidget.posestPopup = widget;
            widget.parent = parentWidget;
        } else
            widget.label = widget.defaultLabel + parentWidget.currentRow.NMPRODUTO;

        return widget;
    };

    this.getPosicaoDeEstoquePopUpObject = function() {
    	return {
            "name": "posestPopup",
            "defaultLabel": "Posição de Estoque do Produto ",
            "template": "widget/form.html",
            "isVisible": true,
            "showAsTab": false,
            "actions": [
                {
                    "name": "back",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup()"
                        }
                    ]
                },
                {
                    "name": "refresh",
                    "label": "Atualizar",
                    "showAsAction": "never",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ValidacoesEstoque_Service.getDataForPosicaoDeEstoquePopup(args.owner.widget)"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "name": "CDALMOXARIFE",
                    "label": "Almoxarifado",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "CDLOCALESTOQ",
                    "label": "Localização",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "NRLOTEESTQ",
                    "label": "Lote",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "NRSUBLOTE",
                    "label": "Sublote",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "QTESTOQDIA",
                    "label": "Quantidade",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "DTVALILOTE",
                    "label": "Data de Validade",
                    "isVisible": true,
                    "sequenceInForm": 80,
                    "readOnly": true,
                    "defaultValue": null,
                    "template": "field/calendar.html",
                    "dateFormat": "DD/MM/YYYY",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "VRCUSTOPROD",
                    "label": "Custo",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "showSymbol": true,
                                    "precision": 3,
                                    "thousands": ",",
                                    "decimal": "."
                            }
                    },
                    "events": []
                    
                },
                
            ]
        };
    };
    
    this.getPosicaoDeEstoquePorProduto = function(row) {
	    var filter = [
	    	{
	    		"name": "CDPRODUTO",
				"operator": "=",
				"value": row.CDPRODUTO
			},
			{
	    		"name": "CDFILIAL",
				"operator": "=",
				"value": row.CDFILIAL
			},
			{
	    		"name": "CDALMOXARIFE",
				"operator": "=",
				"value": row.CDALMOXARIFE
			},
			{
	    		"name": "CDLOCALESTOQ",
				"operator": "=",
				"value": row.CDLOCALESTOQ
			},
			{
	    		"name": "NRLOTEESTQ",
				"operator": "=",
				"value": row.NRLOTEESTQ 
			},
			{
	    		"name": "NRSUBLOTE",
				"operator": "=",
				"value": row.NRSUBLOTE
			}
		];

	    return ValidacoesEstoque_Repository.getPosicaoDeEstoquePorProduto(filter).then(function(success){
	    	return success.dataset.posestdia[0];
	    });
	};
};

Configuration (function(ContextRegister) {
    ContextRegister.register('ValidacoesEstoque_Service', ValidacoesEstoque_Service);
});

// FILE: src/controllers/InventarioRotativoDecorator.js
function InventarioRotativoDecorator(ZHPromise, PreRequests) {

    /*
     * Decorator do Inventário Rotativo. Valida toda abertura de telas,
     * impedindo que aquelas que validam estoque sejam abertas caso haja
     * inventário rotativo pendente, assim como nos sistemas Delphi.
     */
    this.apply = function(containerName, container) {
        if (container.validaEstoque === true) {
            // PreRequests.setContainerBeingValidated();
            var deferred = ZHPromise.defer();
            PreRequests.validaInventarioRotativo(container).then(function(resolve) {
                var inventarioValido = resolve.inventarioValido;
                var container = resolve.container;
                var inveRotaContainer = (containerName === "est#ger03302_inventario_rotativo");
                if (inventarioValido !== inveRotaContainer) {
                    container.events = container.events || [];
                    container.events.push({
                        name: "ContainerAfterinit",
                        code: "PreRequests.validaInventarioRotativo(args.owner)"
                    });
                }
            }).finally(function() {
                deferred.resolve(container);
            });
            return deferred.promise;
        }
        else {
            return container;
        }
    };
}

// FILE: src/controllers/Lib00206_InfComplProduto_Controller.js
var Lib00206_InfComplProduto_Controller = function(ScreenService, ZHPromise, templateManager, metaDataFactory, Lib00206_InfComplProduto_Service, eventEngine, MessageService) {

    var infoComplEvents = {
        "InfoComplBeforeConfirm":"beforeConfirmInfoCompl",
        "InfoComplAfterConfirm":"afterConfirmInfoCompl",
        "InfoComplBeforeCancel":"beforeCancelInfoCompl",
        "InfoComplAfterCancel":"afterCancelInfoCompl",
		"InfoComplInvalidStock":"infoComplInvalidStock",
        "InfoComplOnOpenPopup":"infoComplOpenPopup"
    };

    Object.keys(infoComplEvents).forEach(function(eventName) {
        if (!eventEngine.hasEvent(eventName)) {
            eventEngine.registerEvent(eventName, infoComplEvents[eventName]);
        }
    });
    var rowParent;
	var lotAutomatico;

    this.infComplementares = function(widget,rowItem) {
        var defaultParams = {
            ShowLote : true,
            ShowEntr : false,
            Ajuste : false,
            AtuEstoq : true,
            VerQtEst : true,
            DocuDevo : false,
            AfPiloto : false,
            IsEntrada : true,
            DontEditLocals : false
        };

        rowParent = rowItem;
        defaultParams.ShowLote = rowItem.__is_new;

        Util.extend(defaultParams,widget.paramInfoCompl);  
        
        if(!rowParent.IDCNTRESTOQ)   // IDCNTRESTOQ = null ou inexistente produto controlara estoque
            rowParent.IDCNTRESTOQ = 'S';
            
        if(!(rowParent.CDFILIAL && rowParent.CDFILIMOVI && rowParent.CDPRODESTO && rowParent.CDPRODUTO && rowParent.CDARVPROD && rowParent.NMPRODUTO)){
            console.error('Row do Item incompleta');
        } else if (rowParent.IDCNTRESTOQ != 'N') { // IDCNTRESTOQ = null produto controlara estoque
            Lib00206_InfComplProduto_Service.getParamFilial(rowParent.CDFILIMOVI).then(function(paramfilial){
                paramfilial = paramfilial.dataset.paramfilial[0];
                // IDCTRLESTQ = null filial controlara estoque
                if(paramfilial.IDCTRLESTQ != 'N' && (paramfilial.IDUTILLOTE == 'S' || paramfilial.IDUTILALMOX == 'S')){
                    if(paramfilial.IDUTILLOTE == 'S' && widget.container.name == 'dfe11000_lancamento_entrada')
                        defaultParams.ShowEntr = true;
                    this.openInfCompl(widget,defaultParams, paramfilial);
                } else {
                    notOpenPopup(widget, paramfilial);
                }
            }.bind(this));
        } else {
            notOpenPopup(widget, {});
        }

    };

    this.limpaCamposInfCompl = function(widget){
        widget.fields.forEach(function(field){
            field.value(null);
        });
    };

    this.openInfCompl = function(parentWidget,params, paramfilial){
        var popupWidget = createInfComplProdutoPopup(parentWidget);
        delete(rowParent.$error);
        Util.extend(popupWidget.currentRow,rowParent);
        //this.limpaCamposInfCompl(popupWidget);

        Lib00206_InfComplProduto_Service.getDtprocessa().then(function(success){
            var dtprocessa = success.dataset.paramestoque_processamento[0].DTPROCESSA;

            if(!rowParent.DTLANCMOVI)
                rowParent.DTLANCMOVI = dtprocessa;

            popupWidget.getField('DTVALILOTE').minDate = dtprocessa;

            var filialFilter = {
                "name":"CDFILIAL",
                "operator":"=",
                "value":rowParent.CDFILIMOVI
            };

            ['DSALMOXARIFE', 'DSLOCALESTOQ'].forEach(function(fieldName) {
                var fieldFilter = popupWidget.getField(fieldName).dataSourceFilter;
                var filterIndex = fieldFilter.findIndex(function(element) {return element.name == "CDFILIAL";});

                if(filterIndex == -1)
                    fieldFilter.push(filialFilter);
                else
                    fieldFilter[filterIndex] = filialFilter;

                if (['DSLOCALESTOQ'].includes(fieldName)) {
                    filterIndex = fieldFilter.findIndex(function(element) { return element.name == "DSALMOXARIFE"; });

                    if(filterIndex == -1) {
                        var almoxariFilter = {
                            "name":"CDALMOXARIFE",
                            "operator":"=",
                            "value":"@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('"+parentWidget.name+"','CDALMOXARIFE')@"
                        };

                        fieldFilter.push(almoxariFilter);
                    }
                }
            });

            this.setFieldLote(popupWidget,parentWidget,params);

            if (!params.AtuEstoq) {
                popupWidget.getField('DSALMOXARIFE').setValue(' ');
                popupWidget.getField('DSLOCALESTOQ').setValue(' ');
                popupWidget.getField('NRSUBLOTE').setValue(' ');
                popupWidget.getField('NRLOTEESTQ').setValue(' ');
            }

            popupWidget.params = params;
            popupWidget.getField('IDNOVOLOTE').value(params.ShowEntr);
            popupWidget.getField('IDNOVOLOTE').defaultValue = params.ShowEntr;
            popupWidget.getField('IDNOVOLOTE').readOnly = !params.ShowEntr;
            this.setReadOnlyField(popupWidget,params).then(function(paramfilial){
                Lib00206_InfComplProduto_Service.getProdFili(rowParent.CDPRODUTO,rowParent.CDFILIMOVI).then(function(prodfili){
                    prodfili = prodfili.dataset.prodfili[0];

                    var almoxField = popupWidget.getField('DSALMOXARIFE'),
                        localField = popupWidget.getField('DSLOCALESTOQ'),
                        selectLote = popupWidget.getField('IDNOVOLOTE'),
                        loteField = popupWidget.getField('NRLOTEESTQ'),
                        subloteField = popupWidget.getField('NRSUBLOTE');

                    loteField.validations.required = true;
                    subloteField.setValue(' ');

                    if(prodfili.IDUTCTRLLOTE == 'N' || paramfilial.IDUTILLOTE == 'N') {
                        delete loteField.validations.required;
                        loteField.readOnly = true;
                        loteField.setValue(' ');
                        selectLote.readOnly = true;
                        popupWidget.getField('IDNOVOLOTE').isVisible = false;
                        popupWidget.getField('NRLOTEESTQ').isVisible = false;
                        popupWidget.getField('NRSUBLOTE').isVisible = false;
                    } else{
                        popupWidget.getField('IDNOVOLOTE').isVisible = true;
                        popupWidget.getField('NRLOTEESTQ').isVisible = true;
                        popupWidget.getField('NRSUBLOTE').isVisible = true;
                        
                        if(!parentWidget.currentRow.NRLOTEESTQ) {
                            loteField.value(null);

                            if(prodfili && params.ShowLote) {
                                Lib00206_InfComplProduto_Service.getProximoLote(rowParent.CDFILIMOVI, rowParent.CDPRODESTO).then(function(success){
                                    loteField.value(success.dataset.prox_lote.NRLOTEESTQ);
                                });
                            }
                        }
                    }

                    if(prodfili.IDCNTRESTOQ == 'N'){
                        loteField.readOnly = true;
                        loteField.setValue(' ');
                        subloteField.readOnly = true;
                        subloteField.setValue(' ');
                        almoxField.readOnly = true;
                        almoxField.setValue(' ');
                        delete almoxField.validations.required;
                        localField.readOnly = true;
                        localField.setValue(' ');
                        delete localField.validations.required;
                    }

                    //popupWidget.currentRow.CDALMOXARIFE = !almoxField.readOnly ? parentWidget.currentRow.CDALMOXARIFE||prodfili.CDALMOXARIFE||paramfilial.CDALMOXARIFE||null:' ';
                    //popupWidget.currentRow.CDLOCALESTOQ = !localField.readOnly ? parentWidget.currentRow.CDLOCALESTOQ||prodfili.CDLOCALESTOQ||paramfilial.CDLOCALESTOQ||null:' ';
					if(!almoxField.readOnly || params.DontEditLocals){
                        if(parentWidget.currentRow.CDALMOXARIFE){
                            popupWidget.currentRow.CDALMOXARIFE = parentWidget.currentRow.CDALMOXARIFE;
                        }else if(prodfili.CDALMOXARIFE && paramfilial.IDUTILALMOX == 'S'){
                            popupWidget.currentRow.CDALMOXARIFE = prodfili.CDALMOXARIFE;
                        }else if(paramfilial.CDALMOXARIFE && paramfilial.IDUTILALMOX == 'S'){
                            popupWidget.currentRow.CDALMOXARIFE = paramfilial.CDALMOXARIFE;
                        }else{
                            popupWidget.currentRow.CDALMOXARIFE = ' ';
                        }
                    }else{
                        popupWidget.currentRow.CDALMOXARIFE = ' ';
                    }
                    
                    if(!localField.readOnly || params.DontEditLocals){
                        if(parentWidget.currentRow.CDLOCALESTOQ){
                            popupWidget.currentRow.CDLOCALESTOQ = parentWidget.currentRow.CDLOCALESTOQ;
                        }else if(prodfili.CDLOCALESTOQ && paramfilial.IDUTILLCESTQ == 'S'){
                            popupWidget.currentRow.CDLOCALESTOQ = prodfili.CDLOCALESTOQ;
                        }else if(paramfilial.CDLOCALESTOQ && paramfilial.IDUTILLCESTQ == 'S'){
                            popupWidget.currentRow.CDLOCALESTOQ = paramfilial.CDLOCALESTOQ;
                        }else{
                            popupWidget.currentRow.CDLOCALESTOQ = ' ';
                        }
                    }else{
                        popupWidget.currentRow.CDLOCALESTOQ = ' ';
                    }
                    
                    almoxField.reload().then(function(data){
                        if(popupWidget.currentRow.CDALMOXARIFE){
                            var almoxData = data.dataset.almoxarifado.filter(function(field){
                                return field.CDALMOXARIFE == popupWidget.currentRow.CDALMOXARIFE;
                            }.bind(this));
                            
                            if(almoxData.length != 0){
                                popupWidget.currentRow.DSALMOXARIFE = almoxData[0].DSALMOXARIFE;
                                almoxField.setValue(almoxData[0].DSALMOXARIFE);
                            }else{
                                almoxField.setValue(' ');
                                popupWidget.currentRow.DSALMOXARIFE = ' ';
                                popupWidget.currentRow.CDALMOXARIFE = ' ';
                            }
                        
                            if(popupWidget.currentRow.CDLOCALESTOQ){
                                localField.reload().then(function(data){
                                    localField.setValue(popupWidget.currentRow.CDLOCALESTOQ);
                                    var localData = data.dataset.localestoque.filter(function(field){
                                        return field.CDLOCALESTOQ == popupWidget.currentRow.CDLOCALESTOQ;
                                    }.bind(this));
                                    if(localData.length != 0){
                                        popupWidget.currentRow.DSLOCALESTOQ = localData[0].DSLOCALESTOQ;
                                        localField.setValue(localData[0].DSLOCALESTOQ);
                                    }else{
                                        localField.setValue(' ');
                                        popupWidget.currentRow.DSLOCALESTOQ = ' ';
                                        popupWidget.currentRow.CDLOCALESTOQ = ' ';
                                    }
                                }.bind(this));
                            }
                        }
                    }.bind(this));
                });
            });
            ScreenService.openPopup(popupWidget).then(function(){
                if(parentWidget.infoComplOpenPopup)
                    parentWidget.infoComplOpenPopup();
            }.bind(this));
            
        }.bind(this));
    };

    this.setFieldLote = function(popupWidget,parentWidget,params){
        var fieldLote = popupWidget.getField('NRLOTEESTQ');
        fieldLote.dataSourceFilter=[
                {
                    "name":"P_CDFILIAL",
                    "operator":"=",
                    "value":rowParent.CDFILIMOVI
                },{
                    "name":"P_CDPRODUTO",
                    "operator":"=",
                    "value":rowParent.CDPRODESTO
                },{
                    "name":"P_NRLOTEESTQ",
                    "operator":"=",
                    "value":" "
                },{
                    "name":"P_NRSUBLOTE",
                    "operator":"=",
                    "value":" "
                },{
                    "name":"P_DTPOSICAO",
                    "operator":"=",
                    "value":rowParent.DTLANCMOVI
                }
            ];
        if(!params.ShowEntr || !params.Ajuste ){
            fieldLote.dataSource.name = 'peblib#/lote_infCompl_almox';
            fieldLote.dataSourceFilter = fieldLote.dataSourceFilter.concat([
                {
                    "name":"P_CDALMOXARIFE",
                    "operator":"=",
                    "value":"@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('"+parentWidget.name+"','CDALMOXARIFE')|| ' '@"
                },{
                    "name":"P_CDLOCALESTOQ",
                    "operator":"=",
                    "value":"@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('"+parentWidget.name+"','CDLOCALESTOQ')|| ' '@"
                },{
                    "name":"P_IDQUANTID",
                    "operator":"=",
                    "value":"S"
                }
            ]);
        }else{
            if(params.VerQtEst){
                fieldLote.dataSource.name = 'peblib#/lote_infCompl_almox';
                fieldLote.dataSourceFilter = fieldLote.dataSourceFilter.concat([
                    {
                        "name":"P_CDALMOXARIFE",
                        "operator":"=",
                    "value":"@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('"+parentWidget.name+"','CDALMOXARIFE')|| ' '@"
                    },{
                        "name":"P_CDLOCALESTOQ",
                        "operator":"=",
                    "value":"@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('"+parentWidget.name+"','CDLOCALESTOQ')|| ' '@"
                    },{
                        "name":"P_IDQUANTID",
                        "operator":"=",
                        "value":"N"
                    }
                ]);
            }else{
                fieldLote.dataSource.name = 'peblib#/lote_infCompl_semAlmox';
            }
        }
    };

    function notOpenPopup(widget, params) {
        if(!params)
            params = {};
            
        Lib00206_InfComplProduto_Service.validateInventory(widget, params).then(function(resolve) {
            var pendencies = resolve.length;
            if(pendencies > 0){
                ScreenService.showMessage('Operação bloqueada! Inventário pendente para a filial.');
                ZHPromise.when(widget.beforeConfirmInfoCompl && widget.AlwaysCallsBeforeEvent !== false ? widget.beforeConfirmInfoCompl() : undefined).then(function(){
                    if(widget.infoComplInvalidStock)
                        widget.infoComplInvalidStock();
                });
            }else{
                ZHPromise.when(widget.beforeConfirmInfoCompl && widget.AlwaysCallsBeforeEvent !== false ? widget.beforeConfirmInfoCompl() : undefined).then(function(){
                    if(widget.afterConfirmInfoCompl && widget.AlwaysCallsAfterEvents !== false)
                        widget.afterConfirmInfoCompl();
                });
            }
        }.bind(this));
    }
    
    var setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            currentRow[outDataValue[key]] = valuesToSetOnCurrentRow[key];
        }
    };

    var locaisValidos = function(widget) {
        var almoxarifado = widget.getField('DSALMOXARIFE');
        var local = widget.getField('DSLOCALESTOQ');
        var nlote = widget.getField('NRLOTEESTQ');
        var utAlmoxarifado = !almoxarifado.readOnly;
        var utLocal = !local.readOnly;
        var utLote = !nlote.readOnly || lotAutomatico;
        var almoxarifadoValido = false; 
        var localValido = false;
        var loteValido = false;
        var locaisValidos = false;
		var DontEditLocals = widget.getParent().InfoCompl.params.DontEditLocals;
        
        var arrLocais = ['CDALMOXARIFE', 'CDLOCALESTOQ', 'NRLOTEESTQ', 'NRSUBLOTE'];
        
        arrLocais.forEach(function(local) {
            if(widget.currentRow[local] == "")
                widget.currentRow[local] = " ";
        }.bind(this));
         
        if((utAlmoxarifado && almoxarifado.value() !== ' ') || (DontEditLocals && almoxarifado.value() !== ' '))
            almoxarifadoValido = true;
        else
        if(!utAlmoxarifado && almoxarifado.value() === ' ')
            almoxarifadoValido = true;
                
        if((utLocal && local.value() !== ' ') || (DontEditLocals && local.value() !== ' '))
            localValido = true;
        else
        if(!utLocal && local.value() === ' ')
            localValido = true;
                
        if(utLote && nlote.value() !== ' ')
            loteValido = true;
        else
        if(!utLote && nlote.value() === ' ')
            loteValido = true;
                
        if(almoxarifadoValido && localValido && loteValido)
            locaisValidos = true;
            
        return locaisValidos;
    };
    
    this.saveLote = function(widget){
        if(widget.isValid() && locaisValidos(widget)){
            var row  = widget.currentRow;
            if(row.DTVALILOTE && row.DTFABRPROD && moment(row.DTFABRPROD,'DD/MM/YYYY').isAfter(moment(row.DTVALILOTE,'DD/MM/YYYY')) && widget.params.IsEntrada){
                MessageService.showMessageByCode("DTFABRICACAO_MENOR_OU_IGUAL_DTVALIDADE");
            }else if(row.DTVALILOTE && moment(row.DTVALILOTE,'DD/MM/YYYY').isBefore(moment(widget.getField('DTVALILOTE').minDate,'DD/MM/YYYY')) && widget.params.IsEntrada){
                MessageService.showMessageByCode("DTVALIDADE_MAIOR_DTPROCESSAMENTO");
            }else{
                var parent = widget.getParent();
                
                //atualiza currentrow com posição de estoque
                var lote =    widget.getField('NRLOTEESTQ');
                var subLote = widget.getField('NRSUBLOTE');
                var loteOutData = lote.outData;
                
                lote.dataSourceFilter.forEach(function(params){
                    if(params.name == "P_NRSUBLOTE")
                        params.value = subLote.value();
                	if(params.name == "P_NRLOTEESTQ")
                	    params.value = lote.value();
                }.bind(this));
                
                Lib00206_InfComplProduto_Service.validateInventory(widget).then(function(resolve) {
                    var pendencies = resolve.length;
                    if(pendencies > 0){
                        MessageService.getMessageByCode("VALIDAINVEROTAALMOX").then(function(almoxPhrase) {
			                var messageAlmox  = ScreenService.i18n(almoxPhrase, {CDALMOX: widget.currentRow.CDALMOXARIFE});
                            if(widget.currentRow.CDALMOXARIFE != ' '  && widget.currentRow.CDLOCALESTOQ != ' '){
			                    MessageService.getMessageByCode("VALIDAINVEROTAALMOX").then(function(localPhrase) {
			                        var messageLocal  = ScreenService.i18n(localPhrase, {CDLOCAL: widget.currentRow.CDLOCALESTOQ});
			                        ScreenService.showMessage(messageAlmox+messageLocal);
			                    }.bind(this));
                            }else if(widget.currentRow.CDALMOXARIFE != ' '  && widget.currentRow.CDLOCALESTOQ == ' '){
                                ScreenService.showMessage(messageAlmox);
                            }else{
                                ScreenService.showMessage('Operação bloqueada! Inventário pendente para a filial.');
                            }
                            
                            if(parent.infoComplInvalidStock)
                                parent.infoComplInvalidStock();
                                
                        }.bind(this));
                    }else{
                        ZHPromise.when(parent.beforeConfirmInfoCompl?parent.beforeConfirmInfoCompl():undefined).then(function(){
                            Lib00206_InfComplProduto_Service.saveLote(row).then(
                            function(){
                                rowParent.CDALMOXARIFE = row.CDALMOXARIFE;
                                rowParent.CDLOCALESTOQ = row.CDLOCALESTOQ;
                                rowParent.NRLOTEESTQ = row.NRLOTEESTQ;
                                rowParent.NRSUBLOTE = row.NRSUBLOTE;
                                rowParent.DTVALILOTE = row.DTVALILOTE;
                                rowParent.QTESTOQDIA = (row.QTESTOQDIA) ? row.QTESTOQDIA : 0;
                                ScreenService.closePopup();
                                parent.activate();
                                lote.reload().then(function(posEst){
                                    if(posEst.dataset.lote_infCompl_almox.length > 0){
                                        var valuesToSet = posEst.dataset.lote_infCompl_almox[0];
                                        setCurrentRowWithOutData(row, valuesToSet, loteOutData);
                                        setCurrentRowWithOutData(rowParent, valuesToSet, loteOutData);
                                    }else{
                                        rowParent.QTESTOQDIA = 0;
                                        rowParent.VRESTOQDIA = 0;
                                    }
                                    if(parent.afterConfirmInfoCompl)
                                        parent.afterConfirmInfoCompl();
                                }.bind(this));
                                
                            },
                            function(error){
                                ScreenService.showMessage(error.data.error);
                            }
                        );
                        });
                    }
                }.bind(this));
            

            }
        }
    };

    var createInfComplProdutoPopup = function(parentWidget) {
        var widget = parentWidget.InfoCompl;
        if(!widget){
            widget = getPopUpObject();
            widget.container = parentWidget.container;
            widget = metaDataFactory.widgetFactory(widget, parentWidget);
            parentWidget.InfoCompl = widget;
            widget.parent = parentWidget;
            return widget;
        }else {
            return widget;
        }
    };

    this.getInfComplFieldVal = function(parentName, fieldName){
        return templateManager.container.getWidget(parentName).InfoCompl.getField(fieldName).value();
    };

    this.setReadOnlyField = function(popupWidget,params){
        return Lib00206_InfComplProduto_Service.getParamFilial(popupWidget.currentRow.CDFILIMOVI).then(function(paramfilial){
            paramfilial = paramfilial.dataset.paramfilial[0];
			lotAutomatico = paramfilial.IDENTRAUTLOTE === 'S';
            var idnovolote = popupWidget.getField('IDNOVOLOTE').value();
            popupWidget.getField('DSALMOXARIFE').readOnly = paramfilial.IDUTILALMOX == 'N' || !params.AtuEstoq || params.DontEditLocals;
            popupWidget.getField('DSLOCALESTOQ').readOnly = paramfilial.IDUTILLCESTQ == 'N' || !params.AtuEstoq || params.DontEditLocals;
            popupWidget.getField('DTVALILOTE').readOnly = paramfilial.IDUTILLOTE == 'N' || !params.ShowEntr || params.DocuDevo || !idnovolote;
            popupWidget.getField('NRLOTEESTQ').readOnly = paramfilial.IDUTILLOTE == 'N' || !params.AtuEstoq || params.DocuDevo || (paramfilial.IDENTRAUTLOTE === 'S' && params.ShowEntr);

            popupWidget.getField('NRSUBLOTE').readOnly    = popupWidget.getField('NRLOTEESTQ').readOnly || !idnovolote || (paramfilial.IDENTRAUTLOTE === 'S' && params.ShowEntr);
            popupWidget.getField('DTFABRPROD').readOnly   = (paramfilial.IDUTILLOTE !== 'S') || !params.ShowEntr || params.DocuDevo || !idnovolote;

            if(paramfilial.IDUTILLOTE ==='S' && idnovolote){
                if(!popupWidget.getField('DTVALILOTE').validations.hasOwnProperty('required'))
                    popupWidget.getField('DTVALILOTE').validations = {"required": {"message": "Campo Obrigatorio"}};
            }else{
                delete popupWidget.getField('DTVALILOTE').validations.required;
            }

            if (!popupWidget.getField('DSALMOXARIFE').readOnly) {
                if (!popupWidget.getField('DSALMOXARIFE').validations.hasOwnProperty('required'))
                    popupWidget.getField('NRLOTEESTQ').validations = { "required": { "message": "Campo Obrigatorio" } };
            }
            else {
                delete popupWidget.getField('DSALMOXARIFE').validations.required;
            }

            if (!popupWidget.getField('DSLOCALESTOQ').readOnly) {
                if (!popupWidget.getField('DSLOCALESTOQ').validations.hasOwnProperty('required'))
                    popupWidget.getField('NRLOTEESTQ').validations = { "required": { "message": "Campo Obrigatorio" } };
            }else {
                delete popupWidget.getField('DSLOCALESTOQ').validations.required;
            }

            if(!popupWidget.getField('NRLOTEESTQ').readOnly){
                if(!popupWidget.getField('NRLOTEESTQ').validations.hasOwnProperty('required'))
                    popupWidget.getField('NRLOTEESTQ').validations = {"required": {"message": "Campo Obrigatorio"}};
            }else{
                delete popupWidget.getField('NRLOTEESTQ').validations.required;
            }


            if(idnovolote){
                popupWidget.getField('NRLOTEESTQ').template = 'field/text-edit.html';
                popupWidget.getField('NRLOTEESTQ').dataSourceFilter[2].value = ' ';
                popupWidget.getField('NRLOTEESTQ').dataSourceFilter[3].value = ' ';
            }else{
                popupWidget.getField('NRLOTEESTQ').template = 'zh-select-autocomplete#field/select-autocomplete.html';
                popupWidget.getField('NRLOTEESTQ').dataSourceFilter[2].value = 'TD';
                popupWidget.getField('NRLOTEESTQ').dataSourceFilter[3].value = 'TD';
            }
            
			var useUniLote = paramfilial.IDENTRUNILOTE == 'S'? true:false;
            this.liberaEdit(popupWidget, !useUniLote);
            popupWidget.reload();

            return paramfilial;
        }.bind(this));
    };

    this.idNovoLoteChange = function(widget){
        this.setReadOnlyField(widget,widget.params);

        widget.getField('NRLOTEESTQ').value(null);
        widget.getField('NRSUBLOTE').value(null);

        if(widget.getField('IDNOVOLOTE').value())
            widget.getField('NRSUBLOTE').setValue(' ');

        widget.getField('DTVALILOTE').value(null);
        widget.getField('DTFABRPROD').value(null);
    };
    
    this.loteOnChange = function(widget){
        var novoLote = widget.getField('IDNOVOLOTE').value();
        
        if(novoLote){
            var nrLote = widget.getField('NRLOTEESTQ');
            nrLote.dataSourceFilter[2].value = nrLote.value();
        }

    };

    this.cancelaLancamento = function(widget){
        var parent = widget.getParent();
        
        //Lib00206_InfComplProduto_Service.getParamFilial(rowParent.CDFILIMOVI).then(function(paramfilial){
            //paramfilial = paramfilial.dataset.paramfilial[0];
                ZHPromise.when(parent.beforeCancelInfoCompl?parent.beforeCancelInfoCompl():undefined).then(function(){
                    ScreenService.closePopup();
                    if(parent.afterCancelInfoCompl)
                        parent.afterCancelInfoCompl();
                });
            
       //});
    };

    this.almoxarifeChange = function(args) {
        var fieldLocalEstoq = args.owner.field.widget.getField('DSLOCALESTOQ');
        var fieldCdLocalEstoq = args.owner.field.widget.getField('CDLOCALESTOQ');
        var fieldLote = args.owner.field.widget.getField('NRLOTEESTQ');
        var fieldSubLote = args.owner.field.widget.getField('NRSUBLOTE');

        if (fieldLocalEstoq.readOnly || fieldLocalEstoq.readOnlyOnEdit){
            fieldCdLocalEstoq.setValue(' ');
            fieldLocalEstoq.setValue(' ');
        }else{
            fieldCdLocalEstoq.setValue('');
            fieldLocalEstoq.setValue('');
        }
        fieldLocalEstoq.reload();
        
        if(fieldLote.readOnly || fieldLote.readOnlyOnEdit)
            fieldLote.setValue(' ');
        else
            fieldLote.setValue('');
            
        fieldLote.reload();
        
        if(fieldSubLote.readOnly || fieldSubLote.readOnlyOnEdit)
            fieldSubLote.setValue(' ');
        else
            fieldSubLote.setValue('');
            
    };
	this.localChange = function(args){
        
        var fieldLote = args.owner.field.widget.getField('NRLOTEESTQ');
        var fieldSubLote = args.owner.field.widget.getField('NRSUBLOTE');
        
        if(!lotAutomatico){
            if(fieldLote.readOnly || fieldLote.readOnlyOnEdit)
                fieldLote.setValue(' ');
            else
                fieldLote.setValue('');
        }
            
        fieldLote.reload();
        
        if(fieldSubLote.readOnly || fieldSubLote.readOnlyOnEdit)
            fieldSubLote.setValue(' ');
        else
            fieldSubLote.setValue('');
    };

	this.liberaEdit = function(widget, status){
	    if(!status)
	        status = false;
	   
        var actionEdit = widget.getAction('liberar');
        actionEdit.readOnly = status;
        actionEdit.isVisible = !status;
    };
    
    this.liberaJustificativa = function(widget){
        var fieldJustific = widget.getField('DSJUSTIFICATIVA');
        fieldJustific.validations = {"required": {"message": "Campo Obrigatorio"}};
        fieldJustific.isVisible = true;
        widget.getField('NRLOTEESTQ').readOnly = false;
        widget.getField('NRSUBLOTE').readOnly = false;
    };

    var getPopUpObject = function() {
        return {
            "id": "000",
            "isVisible": true,
            "name": "infComplProduto",
            "label": "Informações Complementares do Produto",
            "template": "widget/form.html",
            "currentRow": {},
            "showAsTab": false,
            "actions": [
                {
                    "id": 1,
                    "name": "cancelAction",
                    "label": "Cancelar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [
                        {
                            "id": 1,
                            "name": "ActionEvent",
                            "code": "Lib00206_InfComplProduto_Controller.cancelaLancamento(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": 4,
                    "name": "confirmAction",
                    "label": "Confirmar",
                    "showAsAction": "never",
                    "isVisible": true,
                    "events": [
                        {
                            "id": 1,
                            "name": "ActionEvent",
                            "code": "Lib00206_InfComplProduto_Controller.saveLote(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": 1,
                    "name": "liberar",
                    "label": "Liberar Lote",
                    "isVisible": true,
                    "readOnly": true,
                    "showAsAction": "view|edit|always",
                    "icon": "key",
                    "events": [
                        {
                            "id": 1,
                            "name": "ActionEvent",
                            "code": "Lib00206_InfComplProduto_Controller.liberaJustificativa(args.owner.widget)"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "name": "NMPRODUTO",
                    "label": "Produto",
                    "isVisible": true,
                    "readOnly": true,
                    "readOnlyOnEdit": true,
                    "class": "6",
                    "showOnForm": true,
                    "showOnList": true,
                    "showOnView": true,
                    "defaultValue": null,
                    "descriptionOrder": 1,
                    "template": "field/text-edit.html"
                },
                {
                    "name": "CDARVPROD",
                    "label": "Cod. Produto",
                    "isVisible": true,
                    "readOnly": true,
                    "readOnlyOnEdit": true,
                    "class": "6",
                    "showOnForm": true,
                    "showOnList": true,
                    "showOnView": true,
                    "defaultValue": null,
                    "descriptionOrder": 1,
                    "template": "field/text-edit.html"
                },
                {
                    "id": 0,
                    "name": "CDALMOXARIFE",
                    "label": "Almoxarifado",
                    "template": "field/text-edit.html",
                    "isVisible": false
                },
                {
                    "id": 0,
                    "name": "DSALMOXARIFE",
                    "label": "Almoxarifado",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "showOnView": true,
                    "class": 6,
                    "order": "CDALMOXARIFE",
                    "template": "zh-select-autocomplete#field/select-autocomplete.html",
                    "searchableFields": [
                        "CDALMOXARIFE",
                        "DSALMOXARIFE"
                    ],
                    "displayedFields": [
                        "CDALMOXARIFE",
                        "DSALMOXARIFE"
                    ],
                    "showSelectAs": "grid",
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "peblib#/crud/almoxarifado"
                    },
                    "dataSourceFilter": [
                        {
                            "name": "CDALMOXARIFE",
                            "operator": "<>",
                            "value": " "
                        }
                    ],
                    "descriptionField": "DSALMOXARIFE",
                    "valueField": "CDALMOXARIFE",
                    "outData": {
                        "CDALMOXARIFE": "CDALMOXARIFE",
                        "DSALMOXARIFE": "DSALMOXARIFE"
                    },
                    "fields": [
                        {
                            "label": "Código",
                            "name": "CDALMOXARIFE",
                            "showOnList": true,
							"template": "field/text-edit.html",
                            "isVisible": true
                        },
                        {
                            "label": "Almoxarifado",
                            "name": "DSALMOXARIFE",
                            "showOnList": true,
							"isVisible": true
                        }
                    ],
                    "events": [
						{
                            "name": "FieldOnClick",
                            "code": "args.owner.field.initializeField();"
                        },
                        {
                            "name": "FieldBeforeSelectOpen",
                            "code": "args.owner.field.initializeField();"
                        },
                        {
                            "name": "FieldOnChange",
                            "code": "Lib00206_InfComplProduto_Controller.almoxarifeChange(args);"
                        }
                    ],
                    "validations": {
                        "required": {
                            "message": "Campo obrigatório"
                        }
                    }
                },
                {
                    "id": 0,
                    "name": "CDLOCALESTOQ",
                    "label": "Localização",
                    "template": "field/text-edit.html",
                    "isVisible": false
                },
                {
                    "id": 0,
                    "name": "DSLOCALESTOQ",
                    "label": "Localização",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": 6,
                    "order": "CDLOCALESTOQ",
                    "template": "zh-select-autocomplete#field/select-autocomplete.html",
                    "searchableFields": [
                        "CDLOCALESTOQ",
                        "DSLOCALESTOQ"
                    ],
                    "displayedFields": [
                        "CDLOCALESTOQ",
                        "DSLOCALESTOQ"
                    ],
                    "showSelectAs": "grid",
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "peblib#/crud/localestoque"
                    },
                    "dataSourceFilter": [
                        {
                            "name": "CDLOCALESTOQ",
                            "operator": "<>",
                            "value": " "
                        }
                    ],
                    "descriptionField": "DSLOCALESTOQ",
                    "valueField": "CDLOCALESTOQ",
                    "outData": {
                        "CDLOCALESTOQ" : "CDLOCALESTOQ",
                        "DSLOCALESTOQ" : "DSLOCALESTOQ"
                    },
                    "fields": [
                        {
                            "label": "Código",
                            "name": "CDLOCALESTOQ",
                            "showOnList": true
                        },
                        {
                            "label": "Descrição",
                            "name": "DSLOCALESTOQ",
                            "showOnList": true
                        }
                    ],
                    "events": [
                        {
                            "name": "FieldOnClick",
                            "code": "args.owner.field.initializeField();"
                        },
						{
                            "name": "FieldOnChange",
                            "code": "Lib00206_InfComplProduto_Controller.localChange(args);"
                        }
                    ],
                    "validations": {
                        "required": {
                            "message": "Campo obrigatório"
                        }
                    }
                },
                {
                    "id": 3,
                    "name": "IDNOVOLOTE",
                    "label": "Lote",
                    "isVisible": true,
                    "readOnly": false,
                    "filterProperties": {
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "dataSource": {
                        "data": [
                            {
                                "IDNOVOLOTE": false,
                                "DSNOVOLOTE": "Lote Existente"
                            },
                            {
                                "IDNOVOLOTE": true,
                                "DSNOVOLOTE": "Novo Lote"
                            }
                        ]
                    },
                    "valueField": "IDNOVOLOTE",
                    "descriptionField": "DSNOVOLOTE",
                    "hideEmptyValue": true,
                    "template": "field/select-native.html",
                    "class": 2,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "placeholder": null,
                    "events": [
                        {
                            "name": "FieldOnChange",
                            "code": "Lib00206_InfComplProduto_Controller.idNovoLoteChange(args.owner.field.widget)"
                        },
						{
                            "name": "FieldBeforeSelectOpen",
                            "code": "args.owner.field.reload()"
                        },
                        {
                            "name": "FieldBeforeSelectOpen",
                            "code": "args.owner.field.reload()"
                        }
                    ]
                },
                {
                    "id": 0,
                    "name": "NRLOTEESTQ",
                    "label": "Lote",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": 5,
                    "order": "NRLOTEESTQ",
                    "template": "zh-select-autocomplete#field/select-autocomplete.html",
                    "searchableFields": [
                        "NRLOTEESTQ",
                        "DTVALILOTE"
                    ],
                    "displayedFields": [
                        "NRLOTEESTQ",
                        "DTVALILOTE",
                        "QTESTOQDIA"
                    ],
                    "showSelectAs": "grid",
                    "maxlength": 20,
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "peblib#/lote_infCompl_semAlmox"
                    },
                    "dataSourceFilter": [],
                    "descriptionField": "NRLOTEESTQ",
                    "valueField": "NRLOTEESTQ",
                    "outData": {
                        "NRLOTEESTQ": "NRLOTEESTQ",
                        "NRSUBLOTE": "NRSUBLOTE",
                        "DTVALILOTE": "DTVALILOTE",
                        "QTESTOQDIA": "QTESTOQDIA",
                        "VRESTOQDIA": "VRESTOQDIA",
                        "CDFABRIC": "CDFABRIC"
                    },
                    "fields": [
                        {
                            "label": "Lote",
                            "name": "NRLOTEESTQ",
                            "showOnList": true
                        },
                        {
                            "label": "Sublote",
                            "name": "NRSUBLOTE",
                            "showOnList": true
                        },
                        {
                            "id": "764527517734593004307",
                            "label": "Validade",
                            "name": "DTVALILOTE",
                            "template": "field/calendar.html",
                            "showOnList": true,
                            "displayFormat": "DD/MM/YYYY",
                            "dateFormat": "DD/MM/YYYY"
                        },
                        {
                            "label": "Almoxarifado",
                            "name": "CDALMOXARIFE",
                            "showOnList": true
                        },
                        {
                            "label": "Localização",
                            "name": "CDLOCALESTOQ",
                            "showOnList": true
                        },
                        {
                            "label": "Quantidade",
                            "name": "QTESTOQDIA",
                            "showOnList": true,
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "precision": 2,
                                    "showSymbol": false
                                }
                            }
                        },
                        {
                            "label": "Valor Total",
                            "name": "VRESTOQDIA",
                            "showOnList": true,
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "precision": "2"
                                }
                            }
                        }
                    ],
                    "events": [
                        {
                            "name": "FieldOnClick",
                            "code": "args.owner.field.initializeField();"
                        },
						{
                            "name": "FieldBeforeSelectOpen",
                            "code": "args.owner.field.reload()"
                        },
                        {
                            "name": "FieldOnChange",
                            "code": "Lib00206_InfComplProduto_Controller.loteOnChange(args.owner.field.widget)"
                        }
                    ],
                    "validations": {
                         "required": {
                            "message": "Campo obrigatório"
                        }
                    }
                },
                {
                    "id": 3,
                    "name": "NRSUBLOTE",
                    "label": "Sublote",
                    "isVisible": true,
                    "readOnly": false,
                    "defaultValue": null,
                    "maxlength": 4,
                    "order": "NRSUBLOTE",
                    "filterProperties": {
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 5,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": [],
                    "validations": {}
                },
                {
                    "id": 7,
                    "name": "DTVALILOTE",
                    "label": "Data de Validade",
                    "isVisible": true, 
                    "readOnly": true,
                    "defaultValue": null,
                    "filterProperties": {
                        "template": "field/interval.html",
                        "readOnly": false
                    },
                    "template": "field/calendar.html",
                    "dateFormat": "DD/MM/YYYY",
                    "class": 6,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "validations": {
                        "required": {
                            "message": "Campo obrigatório"
                        }
                    },
                    "events": [
                        {
                            "name": "FieldOnChange",
                            "code": "args.owner.field.widget.activate();"
                        }
                    ]
                },
                {
                    "id": 7,
                    "name": "DTFABRPROD",
                    "label": "Data de Fabricação",
                    "isVisible": true,
                    "sequenceInForm": 90,
                    "readOnly": false,
                    "defaultValue": null,
                    "filterProperties": {
                        "template": "field/interval.html"
                    },
                    "template": "field/calendar.html",
                    "dateFormat": "DD/MM/YYYY",
                    "class": 6,
                    "descriptionOrder": 4,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": []
                },
				{
                    "id": 7,
                    "name": "DSJUSTIFICATIVA",
                    "label": "Justificativa",
                    "isVisible": false,
                    "readOnly": false,
                    "defaultValue": null,
                    "maxlength": 50,
                    "filterProperties": {
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 12,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": [],
                    "validations": {}
                }
            ],
            "events": [
                {
                    "name": "WidgetOnEnter",
                    "code": "args.owner.getField('DSALMOXARIFE').reload();"
                }
            ]
        };
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00206_InfComplProduto_Controller', Lib00206_InfComplProduto_Controller);
});

// FILE: src/controllers/Lib00207_PosicaoEstoque_Controller.js
var Lib00207_PosicaoEstoque_Controller = function(ScreenService,ZHPromise,metaDataFactory,Lib00207_PosicaoEstoque_Service,eventEngine) {

    
    this.posicaoEstoque = function(widget, rowItem) {
        var rowParent = rowItem;
        var popUp = this.getPosicaoDeEstoquePopUpObject();
        popUp.label = popUp.defaultLabel + rowItem.NMPRODUTO;
        
        if(!(rowParent.CDFILIAL && rowParent.CDPRODUTO && rowParent.CDARVPROD && rowParent.NMPRODUTO)){
            console.error('Row do Item incompleta');
        }
        else {
          this.openPosicaoDeEstoque(widget, rowItem);
        }
    };
    
    this.openPosicaoDeEstoque = function(parentWidget, rowItem) {
        var widget = this.getPosicaoDeEstoquePopUpObject();
        widget.container = parentWidget.container;
        widget = metaDataFactory.widgetFactory(widget,parentWidget);
        parentWidget.PosEstoq = widget;
        widget.parent = parentWidget;
        widget.currentRow = rowItem;
        widget.label = widget.defaultLabel + rowItem.NMPRODUTO;
		ScreenService.openPopup(widget);
        this.getDataForPosicaoDeEstoquePopup(widget);
    };
    
    this.getPosicaoDeEstoquePopUpObject = function() {
    	return {
            "name": "posestPopup",
            "defaultLabel": "Posição de Estoque do Produto ",
            "template": "widget/form.html",
            "isVisible": true,
            "showAsTab": false,
            "actions": [
                {
                    "name": "back",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup()"
                        }
                    ]
                },
                {
                    "name": "refresh",
                    "label": "Atualizar",
                    "showAsAction": "never",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "Lib00207_PosicaoEstoque_Controller.getDataForPosicaoDeEstoquePopup(args.owner.widget)"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "name": "CDALMOXARIFE",
                    "label": "Almoxarifado",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "CDLOCALESTOQ",
                    "label": "Localização",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "NRLOTEESTQ",
                    "label": "Lote",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "NRSUBLOTE",
                    "label": "Sublote",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "QTESTOQDIA",
                    "label": "Quantidade",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "DTVALILOTE",
                    "label": "Data de Validade",
                    "isVisible": true,
                    "sequenceInForm": 80,
                    "readOnly": true,
                    "defaultValue": null,
                    "template": "field/calendar.html",
                    "dateFormat": "DD/MM/YYYY",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                }
            ]
        };
    };
    
    this.getDataForPosicaoDeEstoquePopup = function(popup) {
    	Lib00207_PosicaoEstoque_Service.getPosicaoDeEstoquePorProduto(popup.currentRow).then(function(posestdia) {
    		if(posestdia) {
	    		popup.currentRow = posestdia;
	    		popup.parent.currentRow.QTESTOQDIA = posestdia.QTESTOQDIA;
    		}
    	}).catch(function(error) {
    		ScreenService.showMessage(error.data.error);
    	});
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00207_PosicaoEstoque_Controller', Lib00207_PosicaoEstoque_Controller);
});

// FILE: src/controllers/PreRequests.js
/*
 * Classe que tem o objetivo de executar validações gerais em
 * telas e requisições. São utilizados três elementos: eventAggregator,
 * interceptor (restEngine) e Decorator. O eventAggregator é executado
 * ao iniciar ou finalizar requisições; o interceptor é executado antes
 * de iniciar as requisições, fazendo validações antes de permitir que
 * as mesmas prossigam; e o Decorator é executado antes de abrir qualquer
 * container, alterando-o antes de ser aberto.
 */
var PreRequests = function (ContainerDecoratorProvider, eventAggregator, ZHPromise, InventarioRotativo_Service, ScreenService, restEngine, templateManager) {

	var containerBeingValidated = 0;
	var requestSent = false;
	var self = this;

	/*
	 * Função para indicar que um container está sendo validado, impedindo
	 * que o interceptor sobrescreva as validações feitas pelo Decorator.
	 * É utilizado um valor numérico para que, caso haja mais de um Decorator
	 * sendo utilizado, não haja conflito de condição de corrida.
	 */
	self.setContainerBeingValidated = function () {
		containerBeingValidated++;
	};

	/*
	 * Função utilizado no final de uma validação de Decorator, para que
	 * o interceptor possa voltar a validar as requisições quando a variável
	 * chegar ao valor 0 (quando não houver validação de qualquer Decorator).
	 */
	self.freeContainerBeingValidated = function () {
		containerBeingValidated--;
	};

	/*
	 * Função para verificar se há algum Decorator validando um
	 * container. Caso haja, o número na variável 'containerBeingValidated'
	 * deve ser maior que 0, retornando true.
	 */
	self.getContainerIsBeingValidated = function () {
		return containerBeingValidated > 0;
	};

	/*
	 * 	Função que executa a validação de inventário rotativo ao entrar
	 * 	em telas que estão parametrizadas para realizar tal validação.
	 * 	Se houver inventário rotativo pendente para a unidade, será exibida
	 * 	uma mensagem e haverá o redirecionamento para uma página em branco,
	 * 	necessária para impedir o acesso à tela acessada.
	 */
	self.validaInventarioRotativo = function (container) {
		if (container.validaEstoque === true) {
			self.setContainerBeingValidated();
			var deferred = ZHPromise.defer();
			var containerName = container.name;
			InventarioRotativo_Service.checkInventarioPendente().then(function(inventarioValido){
				var inveRotaContainer = (containerName === "ger03302_inventario_rotativo");
				if (!(inventarioValido || inveRotaContainer)) {
					container.widgets = [];
					container.events = [];
					ScreenService.showMessage("Operação bloqueada! Existe inventário pendente para a unidade.").then(function(){
						ScreenService.openWindow("est#ger03302_blank_window");
					});
				} else if (inveRotaContainer && inventarioValido) {
					container.widgets = [];
					ScreenService.showMessage(ScreenService.i18n("Não há inventário pendente para a unidade.")).then(function(){
						ScreenService.openWindow("est#ger03302_blank_window");
					});
				} else if (inveRotaContainer) {
					container.validaEstoque = false;
				}
				deferred.resolve({
					container: container,
					inventarioValido: inventarioValido
				});
			}.bind(this)).catch(function (error) {
				container.widgets = [];
				container.events = [];
				ScreenService.showMessage("Houve um erro ao tentar conectar com o servidor.").then(function(){
					ScreenService.openWindow("est#ger03302_blank_window");
				});
				deferred.reject(error);
			}).finally(function(){
				self.freeContainerBeingValidated();
			});
			return deferred;
		}
	};

	/*
	 * Função que envia nos headers de qualquer requisição parâmetros que
	 * indicam se a tela atual valida estoque e qual unidade será validada.
	 * Para isso, o container da tela deve conter as seguinter propriedades:
	 *
	 *   * filialOnRequest - String  - CDFILIAL utilizada no processo.
	 *   Opcional. Buscará a unidade logada como default.
	 *
	 *   * validaEstoque   - Boolean - Indica se a tela verifica estoque.
	 *   Opcional. Valor default é false, não validando inventário rotativo
	 *   automaticamente.
	 */
	eventAggregator.subscribe('EventOnRequestStart', function (eventData) {
		var url = eventData.requestConfig.url;
		// if para impedir de adicionar os headers caso seja uma requisição
		// para gerar relatórios
		if (url.indexOf('generateReport') === -1 && url.indexOf('generateXLS') === -1) {
			var container = templateManager.container;
			var filial = false;
			var validaEstoque = false;
			if (container) {
				filial = container.filialOnRequest || false;
				validaEstoque = (container.validaEstoque !== undefined) ? container.validaEstoque : false;
			}
			eventData.requestConfig.headers.Filial = filial;
			eventData.requestConfig.headers.ValidaEstoque = validaEstoque;
		}
	});

	/*
	 * Registrador do interceptador de requisições do Inventário Rotativo.
	 * Toda requisição pro backend proveniente de telas que validam estoque
	 * será validada antes de ser concretizada. Isso impede o funcionamento
	 * de telas caso haja inventário rotativo pendente, assim como nos
	 * sistemas Delphi.
	 */
	restEngine.registerInterceptor({
		method: "ALL",
		interceptor: function (data) {
			if (!(requestSent || self.getContainerIsBeingValidated())) {
				var container = templateManager.container;
				if (container && container.validaEstoque === true && container.name !== 'ger03302_inventario_rotativo') {
					var deferred = ZHPromise.defer();
					// Variável de controle, para que a requisição da validação
					// realizada aqui não seja verificada.
					requestSent = true;
					InventarioRotativo_Service.checkInventarioPendente().then(function (inventarioIsValid) {
						if (inventarioIsValid) {
							deferred.resolve();
						} else {
							ScreenService.showMessage("Operação bloqueada! Existe inventário pendente para a unidade.");
							deferred.reject();
						}
					}).catch(function () {
						// Caso haja algum erro, impede a requisição.
						deferred.reject();
					}).finally(function () {
						requestSent = false;
					});
					return deferred;
				}
			}
		}
	});

	/*
	 * Função que trabalha a recursividade do mecanismo de preenchimento do
	 * DescriptionField. Recebe uma widget e caso a mesma tenha widgets filhas,
	 * todos os fields do campo "select" dessas será verificado.
	 */
	self.verificaSelectWidgetsAndChilds = function (widget) {
		validaDescriptionFieldSelect(widget);
		if (widget.widgets && widget.widgets.length) {
			widget.widgets.forEach(function (childWidget) {
				self.verificaSelectWidgetsAndChilds(childWidget);
			});
		}
	};

	/*
	 * Função local auxiliar a "verificaSelectWidgetsAndChilds", responsavél por
	 * validar se os campos select de uma widget tem a propriedade description
	 * field preenchida. Caso algum dos fields não tenham a propriedade, a função
	 * 'preencheDescriptionFieldSelect' é executada.
	 */
	function validaDescriptionFieldSelect(widget) {
		if (widget.fields && widget.fields.length) {
			var description;
			widget.fields.forEach(function (field) {
				description = null;
				if (field.isVisible && field.fields && (field.template == "field/select.html" || field.template == "field/select-multiple.html")) {
					if (field.descriptionField) {
						field.fields.forEach(function (childField) {
							if (childField.name == field.descriptionField)
								description = true;
						});
						if (description === null) {
							preencheDescriptionFieldSelect(field);
						}
					} else {
						preencheDescriptionFieldSelect(field);
					}
				}
			});
		}
	}

	/*
	 * Função local auxiliar a "validaDescriptionFieldSelect",
	 * preenche a propriedade "descriptionField" de fields select
	 * com valores default. Ordem de preenchimento: mesmo nome do
	 * field ou primeiro field filho visível.
	 */
	function preencheDescriptionFieldSelect(field) {
		var description = null;
		description = field.fields.filter(function (childField) {
			return childField.isVisible !== false && childField.name == field.name && childField.showOnList;
		});
		field.descriptionField = description && description.length ? description[0].name : null;

		if (!description.length) {
			description = field.fields.filter(function (childField) {
				return childField.isVisible !== false && childField.showOnList;
			});
			field.descriptionField = description && description.length ? description[0].name : null;
		}
	}

	/*
	 * Registrador do Decorator do Inventário Rotativo. Valida toda abertura
	 * de telas, impedindo que aquelas que validam estoque sejam abertas
	 * caso haja inventário rotativo pendente, assim como nos sistemas Delphi.
	 */
	ContainerDecoratorProvider.registerProvider(new InventarioRotativoDecorator(ZHPromise, this));

	/*
	 * Registrador do Decorator de preenchimento default de Description Field.
	 * Valida toda abertura de telas, e aquelas que conterem fields select sem
	 * a propriedade de DescriptionField serão alteradas. O Decorator só é ativado
	 * caso a propriedade "useDecoratorInDescriptionFieldSelectInSelect" tenha valor true.
	 * Normalmente a propriedade é definida no enviroment.js.
	 */
	if (window.useDecoratorInDescriptionFieldSelect)
		ContainerDecoratorProvider.registerProvider(new DescriptionFieldSelectDecorator(ZHPromise, this));
};

Configuration(function (ContextRegister) {
	ContextRegister.register('PreRequests', PreRequests);
});